--[[
****************************************************************************
Input parameters:

--simsim-databases = N: the number of databases to create/use (1 by default)
****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps = 
       {"Desiried transaction throughput (TPS)",-1}
 
}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function thread_init(thread_id)
    local i, j, k, n_fks
    local stmt
    local child_keys, parent_keys
    if sysbench.opt.threads > 1 then
        error("Only one thread is allowed.")
    end
       
    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    con1 = drv:connect()
    con2 = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end

    -- Enum current schema tables
    rs = con:query("select table_name from information_schema.tables " ..
                   "where table_schema = '"..sysbench.opt.mysql_db.."'" ..
                   "and table_type = 'BASE TABLE' and table_name not like '%_ostr'")
    n_table = rs.nrows
    for i = 1, n_table do
        row = rs:fetch_row()
        tables[i] = tostring(row[1])
    end


    -- back up current schema tables
    rs = con:query( "select table_name from information_schema.tables " ..
                    "where table_schema = '"..sysbench.opt.mysql_db.."'" ..
                    "and table_type = 'BASE TABLE' and table_name like '%_ostr'")
    if rs.nrows > 0 then goto skip_backup end

    -- drop RI constraints on back up tables if needed
    rs = con:query( "select table_name, constraint_name from information_schema.TABLE_CONSTRAINTS " ..
                    "where table_schema = '"..sysbench.opt.mysql_db.."' and table_name like '%_ostr' " ..
                    "  and constraint_type = 'FOREIGN KEY'")
    n_fks = rs.nrows
    for i = 1, n_fks do
        row = rs:fetch_row()
        local fk_name = tostring(row[2])
        local table_name = tostring(row[1])
        stmt = ([[alter table %s drop foreign key %s ]]):format(table_name, fk_name)
        ddl_con:query(stmt)
    end

    -- Create back up tables
    for i = 1, n_table do
        ddl_con:query("drop table if exists `"..tables[i].."_ostr` ")
        ddl_con:query("create table `"..tables[i].."_ostr` like "..tables[i])
    end

    -- Create RI constraints
    for i = 1, n_table do
        local rs_fk = con:query("select constraint_name from information_schema.TABLE_CONSTRAINTS " ..
                          "where table_schema = '"..sysbench.opt.mysql_db.."' and table_name = '"..tables[i].."'" ..
                          "  and constraint_type = 'FOREIGN KEY'")
        n_fks = rs_fk.nrows
        for j = 1, n_fks do
            fk = rs_fk:fetch_row()
            fk_name = tostring(fk[1])
            print("Table/FK = "..tables[i] .. fk_name)
            --table[i].fk[j] = fk[1]
            stmt = ([[select group_concat(COLUMN_NAME order by ordinal_position), REFERENCED_TABLE_NAME, 
                             group_concat(REFERENCED_COLUMN_NAME order by ordinal_position) 
                        from information_schema.KEY_COLUMN_USAGE  
                       where constraint_schema = '%s' and constraint_name = '%s']]):format(
                        sysbench.opt.mysql_db, fk_name)
            rs_cols = con1:query(stmt)
            row = rs_cols:fetch_row()
            child_keys = tostring(row[1])
            parent_keys = tostring(row[3])
            parent_table = tostring(row[2])
            rs_conds = con1:query("select update_rule, delete_rule from information_schema.REFERENTIAL_CONSTRAINTS " ..
                                 " where constraint_name = '"..fk_name.."' and constraint_schema = '"..sysbench.opt.mysql_db.."'")
            row = rs_conds:fetch_row()
            local upd_cond = tostring(row[1])
            local del_cond = tostring(row[2])
            -- create FKs on back up tables
            stmt = ([[alter table %s.%s_ostr add constraint %s_ostr 
                        foreign key(%s) references %s_ostr(%s) 
                        on update %s on delete %s ]]):format(sysbench.opt.mysql_db,
            tables[i], fk_name, child_keys, parent_table,
            parent_keys, upd_cond, del_cond)
            print(stmt)
            ddl_con:query(stmt)
        end
    end

    ::skip_backup::
end


-- The main event loop
function event()
    con:query("USE " .. sysbench.opt.mysql_db)
    cur_table = tables[sysbench.rand.uniform(1,n_table)]

    local max_op =  7

    local op_type = sysbench.rand.uniform(1,max_op)
    if op_type == 1 then
      op = "optimize_table"
    elseif op_type == 2 then
        op = "add_pk"
    elseif op_type == 3 then
        op = "drop_pk"
    elseif op_type == 4 then
        op = "change_row_format"
    elseif op_type == 5 then
        op = "add_partitioning"
    elseif op_type == 6 then
        op = "drop_partitioning"
    elseif op_type == 7 then
        op = "change_column"
    elseif op_type == 8 then
        op = "drop_index"
    elseif op_type == 9 then
        op = "set_default"
    elseif op_type == 10 then
        op = "change_auto_inc"
    end
    _G[op](cur_table)
    throttle_event()
end           
   
function optimize_table(table_name)
    print("Optimize table "..table_name)
    ddl_con:query("optimize table "..table_name)
end

function add_pk(table_name)
    local i,j,k
    local stmt
    local key_col = {}
    print("Adding PK for "..table_name)
    -- Looking for existing PK
    rs_pk = con:query("select constraint_name from information_schema.TABLE_CONSTRAINTS " ..
                      "where table_schema = '"..sysbench.opt.mysql_db.."' and table_name = '"..table_name.."'" ..
                      "  and constraint_type = 'PRIMARY KEY'")
    if rs_pk.nrows > 0 then 
        print("Table already has primary key")
        return 
    end
    -- Add PK, we can query the definition out of back up table structure
    rs = con:query("select column_name from information_schema.KEY_COLUMN_USAGE " ..
                      "where table_schema = '"..sysbench.opt.mysql_db.."' and table_name = '"..table_name.."_ostr'" ..
                      "  and constraint_name = 'PRIMARY'")
    if rs.nrows <= 0 then 
        print("Original table does not have primary key")
        return 
    end
    for i = 1, rs.nrows do
        row = rs:fetch_row()
        key_col[i] = tostring(row[1])
    end
    stmt = ([[alter table %s add primary key (%s)]]):format(table_name, table.concat(key_col, ","))
    print(stmt)
    ddl_con:query(stmt)

    -- Add RI : Current table is RI child, we need to make sure the RI parent 
    --          has the correct key so we can add foreign key back

    -- 1. Do we have RI parent table?
    -- 2. Does RI parent table has correct keys. If not, create them.
    -- 3. Create foreign key
    print("Looking for parent keys")
    stmt = ([[select distinct constraint_name
                from information_schema.KEY_COLUMN_USAGE  
                where TABLE_SCHEMA = '%s' and 
                TABLE_NAME = '%s_ostr' and REFERENCED_TABLE_NAME is not NULL and
                COLUMN_NAME = any (select column_name 
                from information_schema.KEY_COLUMN_USAGE 
                where TABLE_SCHEMA = '%s' and table_name = '%s_ostr' and 
                constraint_name = 'PRIMARY')]]):format(sysbench.opt.mysql_db, 
                table_name, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)

    for i = 1, rs.nrows do
        row = rs:fetch_row()
        fk_name = tostring(row[1])
        stmt = ([[select distinct b.table_name, b.index_name from information_schema.key_column_usage a, 
                                                                  information_schema.statistics b 
                   where a.referenced_column_name is null and (a.table_name, a.column_name) in 
                         (select referenced_table_name, referenced_column_name 
                            from information_schema.key_column_usage 
                           where constraint_schema = '%s' and constraint_name = '%s') 
                         and a.TABLE_NAME = b.table_name and a.column_name = b.column_name]]) 
                :format(sysbench.opt.mysql_db, fk_name)
        rs1 = con1:query(stmt)
        for j = 1, rs1.nrows do
            row = rs1:fetch_row()
            t_name = tostring(row[1])
            i_name = tostring(row[2])
            stmt = ([[select index_name from information_schema.statistics where table_schema = '%s' and index_name = '%s' and table_name = '%s']]):format(sysbench.opt.mysql_db, i_name, string.sub(t_name,1,-6))
            rs2 = con2:query(stmt)
            if rs2.nrows == 0 then -- we need create the key
                stmt = ([[select non_unique, group_concat(column_name order by seq_in_index) from information_schema.statistics where table_schema = '%s' and index_name = '%s' and table_name = '%s' group by index_name]]):format(sysbench.opt.mysql_db, i_name, t_name)
                rs2 = con2:query(stmt)
                row = rs2:fetch_row()
                non_unique = tonumber(row[1])
                columns = tostring(row[2])
                if i_name == 'PRIMARY' then 
                    stmt = ([[alter table %s.%s add primary key (%s)]]):format(sysbench.opt.mysql_db, string.sub(t_name,1,-6), columns )
                elseif non_unique == 0 then
                    stmt = ([[create unique index %s.%s on %s ()]]):format(sysbench.opt.mysql_db, t_name, string.sub(t_name,1,-6), columns )
                else
                    stmt = ([[create index %s.%s on %s ()]]):format(sysbench.opt.mysql_db, t_name, string.sub(t_name,1,-6), columns )
                end
                print(stmt)
                ddl_con:query(stmt)
            end
        end

        -- Create the foreign key
        stmt = ([[select constraint_name, table_name, group_concat(column_name order by ordinal_position) child_key, 
                         referenced_table_name, group_concat(referenced_column_name order by ordinal_position) parent_key 
                    from information_schema.KEY_COLUMN_USAGE
                   where TABLE_SCHEMA = '%s' and constraint_name = '%s' 
                   group by table_schema, constraint_name, table_name, referenced_table_name ]]):format(
                       sysbench.opt.mysql_db, fk_name)
        rs1 = con1:query(stmt)
        row = rs1:fetch_row()
        const_name = tostring(row[1])
        table_name = tostring(row[2])
        child_keys = tostring(row[3])
        parent_table = tostring(row[4])
        parent_keys = tostring(row[5])
        stmt = ([[alter table %s.%s add constraint %s foreign key (%s) references %s (%s)]]):format(
                sysbench.opt.mysql_db, string.sub(table_name,1,-6), string.sub(const_name,1,-6), child_keys, string.sub(parent_table,1,-6), parent_keys)
        print(stmt)
        ddl_con:query(stmt)
    end




    -- Add RI : Current PK is RI parent key, we need to make sure the RI child 
    --          has the correct keys and RI constraint.  Rebuild the keys if 
    --          necessary.
    -- 1. Do we have RI child?
    -- 2. Does the children have correct keys?
    -- 3. Create the foreign keys
    print("Looking for child keys")
    stmt = ([[select distinct constraint_name
                from information_schema.KEY_COLUMN_USAGE  
                where REFERENCED_TABLE_SCHEMA = '%s' and 
                REFERENCED_TABLE_NAME = '%s_ostr' and
                REFERENCED_COLUMN_NAME = any (select column_name 
                from information_schema.KEY_COLUMN_USAGE 
                where TABLE_SCHEMA = '%s' and table_name = '%s_ostr' and 
                constraint_name = 'PRIMARY')]]):format(sysbench.opt.mysql_db, 
                table_name, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)

    for i = 1, rs.nrows do
        row = rs:fetch_row()
        fk_name = tostring(row[1])
        -- Find the PK and keys that is referenced in the FK
        stmt = ([[select distinct b.table_name, b.index_name from information_schema.key_column_usage a, 
                                                                  information_schema.statistics b 
                   where a.referenced_column_name is null and (a.table_name, a.column_name) in 
                         (select table_name, column_name 
                            from information_schema.key_column_usage 
                           where constraint_schema = '%s' and constraint_name = '%s') 
                         and a.TABLE_NAME = b.table_name and a.column_name = b.column_name]]) 
                :format(sysbench.opt.mysql_db, fk_name)
        rs1 = con1:query(stmt)
        for j = 1, rs1.nrows do
            row = rs1:fetch_row()
            t_name = tostring(row[1])
            i_name = tostring(row[2])
            stmt = ([[select index_name from information_schema.statistics where table_schema = '%s' and index_name = '%s' and table_name = '%s']]):format(sysbench.opt.mysql_db, i_name, string.sub(t_name,1,-6))
            rs2 = con2:query(stmt)
            if rs2.nrows == 0 then -- we need create the key
                stmt = ([[select non_unique, group_concat(column_name order by seq_in_index) from information_schema.statistics where table_schema = '%s' and index_name = '%s' and table_name = '%s' group by index_name]]):format(sysbench.opt.mysql_db, i_name, t_name)
                rs2 = con2:query(stmt)
                row = rs2:fetch_row()
                non_unique = tonumber(row[1])
                columns = tostring(row[2])
                if i_name == 'PRIMARY' then 
                    stmt = ([[alter table %s.%s add primary key (%s)]]):format(sysbench.opt.mysql_db, string.sub(t_name,1,-6), columns )
                elseif non_unique == 0 then
                    stmt = ([[create unique index %s.%s on %s ()]]):format(sysbench.opt.mysql_db, t_name, string.sub(t_name,1,-6), columns )
                else
                    stmt = ([[create index %s.%s on %s ()]]):format(sysbench.opt.mysql_db, t_name, string.sub(t_name,1,-6), columns )
                end
                print(stmt)
                ddl_con:query(stmt)
            end
        end

        -- Create the foreign key
        stmt = ([[select constraint_name, table_name, group_concat(column_name order by ordinal_position) child_key, 
                         referenced_table_name, group_concat(referenced_column_name order by ordinal_position) parent_key 
                    from information_schema.KEY_COLUMN_USAGE
                   where TABLE_SCHEMA = '%s' and constraint_name = '%s' 
                   group by table_schema, constraint_name, table_name, referenced_table_name ]]):format(
                       sysbench.opt.mysql_db, fk_name)
        rs1 = con1:query(stmt)
        row = rs1:fetch_row()
        const_name = tostring(row[1])
        table_name = tostring(row[2])
        child_keys = tostring(row[3])
        parent_table = tostring(row[4])
        parent_keys = tostring(row[5])
        stmt = ([[alter table %s.%s add constraint %s foreign key (%s) references %s (%s)]]):format(
                sysbench.opt.mysql_db, string.sub(table_name,1,-6), string.sub(const_name,1,-6), child_keys, string.sub(parent_table,1,-6), parent_keys)
        print(stmt)
        ddl_con:query(stmt)
    end
end


function drop_pk(table_name)
    local i
    local stmt
    local key_col = {}
    local t_name, constraint_name
    print("Dropping PK for "..table_name)

    -- If table is partitioned by KEY, we can't drop the PK.
    stmt = ([[select distinct PARTITION_METHOD from information_schema.partitions
               where table_schema = '%s' and table_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs:fetch_row()[1] == 'KEY' then 
        print("Table is partitioned by KEY. Can not drop primary key.")
        return
    end

    -- Looking for existing PK
    rs_pk = con:query("select constraint_name from information_schema.TABLE_CONSTRAINTS " ..
                      "where table_schema = '"..sysbench.opt.mysql_db.."' and table_name = '"..table_name.."'" ..
                      "  and constraint_type = 'PRIMARY KEY'")
    if rs_pk.nrows <= 0 then 
        print("Table does not have primary key")
        return 
    end
    -- Looking for RI child of the primary key
    stmt = ([[select distinct table_name, constraint_name 
                from information_schema.KEY_COLUMN_USAGE  
                where REFERENCED_TABLE_SCHEMA = '%s' and 
                REFERENCED_TABLE_NAME = '%s' and 
                REFERENCED_COLUMN_NAME = any (select column_name 
                from information_schema.KEY_COLUMN_USAGE 
                where TABLE_SCHEMA = '%s' and table_name = '%s' and 
                constraint_name = 'PRIMARY')]]):format(sysbench.opt.mysql_db, 
                table_name, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows > 0 then 
        print("There are RI child depending on the PK. Dropping the foreign key ..")
        for i = 1, rs.nrows do
            row = rs:fetch_row()
            t_name = tostring(row[1])
            constraint_name = tostring(row[2])
            stmt = ([[alter table %s.%s drop foreign key %s]]):format(sysbench.opt.mysql_db, t_name, constraint_name)
            print(stmt)
            ddl_con:query(stmt)
        end
    end

    -- If self is RI child and using the primary key to reference a parent table
    stmt = ([[select distinct constraint_name 
                from information_schema.KEY_COLUMN_USAGE  
                where TABLE_SCHEMA = '%s' and 
                TABLE_NAME = '%s' and REFERENCED_TABLE_NAME is not NULL and
                COLUMN_NAME = any (select column_name 
                from information_schema.KEY_COLUMN_USAGE 
                where TABLE_SCHEMA = '%s' and table_name = '%s' and 
                constraint_name = 'PRIMARY')]]):format(sysbench.opt.mysql_db, 
                table_name, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows > 0 then 
        print("There are RI constraint over the PK. Dropping the foreign key ..")
        for i = 1, rs.nrows do
            row = rs:fetch_row()
            constraint_name = tostring(row[1])
            stmt = ([[alter table %s.%s drop foreign key %s]]):format(sysbench.opt.mysql_db, table_name, constraint_name)
            print(stmt)
            ddl_con:query(stmt)
        end
    end

    stmt = ([[alter table %s.%s drop primary key]]):format(sysbench.opt.mysql_db, table_name)
    print(stmt)
    ddl_con:query(stmt)
end
   

function change_row_format(table_name)
    local i, r_fmt, fl, op, stmt
    -- If table is partitioned by KEY, we can't drop the PK.
    stmt = ([[select distinct ROW_FORMAT from information_schema.tables
               where table_schema = '%s' and table_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    r_fmt = tostring(rs:fetch_row()[1])
    if r_fmt == 'Redundant' then 
        fl = 1
    elseif r_fmt == 'Compact' then 
        fl = 2
    elseif r_fmt == 'Compressed' or r_fmt == 'Dynamic' then 
        fl = 3
    end

    op = sysbench.rand.uniform(fl,4)
    if op == 1 then
        r_fmt = 'Redundant'
    elseif op == 2 then
        r_fmt = 'Compact'
    elseif op == 3 then
        r_fmt = 'Compressed'
    else
        r_fmt = 'Dynamic'
    end

    stmt = ([[alter table %s.%s row_format = %s]]):format(
        sysbench.opt.mysql_db, table_name, r_fmt)
    print(stmt)
    ddl_con:query(stmt)

end

function add_partitioning(table_name)
    local i
    local stmt, part_clause
    print("Adding partitioning to table "..table_name)
    stmt = ([[select partition_name, partition_method, partition_expression from information_schema.partitions
               where table_schema = '%s' and table_name = '%s' and
                     partition_ordinal_position is not null
               order by partition_ordinal_position limit 1]]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows > 0 then 
        print("Table is already partitioned")
        return
    end

    stmt = ([[select partition_method, partition_expression, count(*) from information_schema.partitions 
               where partition_method is not null and table_schema = '%s' and table_name = '%s_ostr'
               group by table_schema, table_name, partition_method]]):format(
                   sysbench.opt.mysql_db, table_name)

    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Original table is not partitioned.")
        return
    end
    row = rs:fetch_row()
    part_method = tostring(row[1])
    part_exp = tostring(row[2])
    part_count = tonumber(row[3])
    print(part_method..part_exp..part_count)
    if part_method == 'KEY' or part_method == 'HASH' or 
       part_method == 'LINEAR KEY' or part_method == 'LINEAR HASH' then
        part_clause = ([[partition by %s (%s) partitions %d]]):format(
            part_method,part_exp,part_count)
    else
        part_clause = ([[partition by %s (%s) (]]):format(part_method,part_exp)
        stmt = ([[select partition_name, PARTITION_DESCRIPTION, PARTITION_COMMENT 
                   from information_schema.partitions
                  where table_schema = '%s' and table_name = '%s_ostr' and
                        partition_ordinal_position is not null
                        order by partition_ordinal_position]]):format(
                sysbench.opt.mysql_db, table_name)

        rs = con:query(stmt)
        for i = 1, rs.nrows do
            row = rs:fetch_row()
            part_name = tostring(row[1])
            part_desc = tostring(row[2])
            part_comment = tostring(row[3])
            part_clause = part_clause .. " partition "..part_name.." values "
            if part_method == "RANGE" or part_method == "RANGE COLUMNS" then
                part_clause = part_clause .. " less than "
            else
                part_clause = part_clause .. " in "
            end
            part_clause = part_clause ..([[(%s) comment = '%s',]]):format(part_desc, part_comment)
        end
        part_clause = string.sub(part_clause,1,-2)..")"
    end
    stmt = ([[alter table %s.%s %s]]):format(sysbench.opt.mysql_db, table_name, part_clause)
    print(stmt)
    ddl_con:query(stmt)

end

function drop_partitioning(table_name)
    local i
    local stmt
    print("Drop partitioning from table "..table_name)
    stmt = ([[select partition_name, partition_method, partition_expression from information_schema.partitions
               where table_schema = '%s' and table_name = '%s' and
                     partition_ordinal_position is not null
               order by partition_ordinal_position limit 1]]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Table is not partitioned")
        return
    end
    stmt = ([[alter table %s.%s remove partitioning]]):format(
        sysbench.opt.mysql_db, table_name)
    print(stmt)
    ddl_con:query(stmt)
end

function change_column(table_name)
    local i
    local stmt
    print("Changing coloumn data type for "..table_name)
    stmt = ([[select column_name, is_nullable, data_type, CHARACTER_MAXIMUM_LENGTH, 
             NUMERIC_PRECISION, NUMERIC_SCALE, COLUMN_COMMENT 
             from information_schema.columns a 
             where a.table_name = '%s'  and a.table_schema = '%s' and a.data_type in ('varchar', 'char') and
             a.column_name not in 
             (select column_name from information_schema.KEY_COLUMN_USAGE 
               where table_schema = '%s' and table_name = '%s')]]):format(
                table_name, sysbench.opt.mysql_db, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Table has no char/varchar type")
        return
    end
            
    pos = sysbench.rand.uniform(1,rs.nrows)
    for i = 1, pos do
        row = rs:fetch_row()
    end
    col_name = tostring(row[1])
    is_null = tostring(row[2])
    data_type = tostring(row[3])
    len = tonumber(row[4])
    comment = tostring(row[7])

    if is_null == 'YES' then
        null_clause = 'NULL'
    else
        null_clause = 'NOT NULL'
    end

    if (comment == 'changed') then
        len = len - 10
        comment = ''
    else
        len = len + 10
        comment = 'changed'
    end
    stmt = ([[alter table %s.%s change column %s %s %s(%d) %s comment '%s']]):format(
        sysbench.opt.mysql_db, table_name, col_name, col_name, data_type, len, null_clause, comment)
    print(stmt)
    ddl_con:query(stmt)
end
    
function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end
    