#!/usr/bin/env sysbench
-- Copyright (C) 2006-2017 Alexey Kopytov <akopytov@gmail.com>

-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

-- ----------------------------------------------------------------------
-- Ganos geometry test case for PolarDB
-- Caution: Data must be prepared before runing the tests.
-- 4 types of testing are executed：
--    * export/outpu
--       ST_AsXXXX
--    * attributes  
--       ST_SRID
--    * spatial relationship
--       ST_Within/ ST_Contains/ ST_Intersects
--    * sptial analyze
--       ST_Intersection/ ST_Buffer
-- Also, types of geometries are included in the dataset, point/linestring/polygons.
-- queries and geometries extented are welcomed.
-- ----------------------------------------------------------------------

require("oltp_common")

sysbench.cmdline.commands.prepare = {
   function ()
      if (not sysbench.opt.auto_inc) then
         -- Create empty tables on prepare when --auto-inc is off, since IDs
         -- generated on prepare may collide later with values generated by
         -- sysbench.rand.unique()
         sysbench.opt.table_size=0
      end

      cmd_prepare()
   end,
   sysbench.cmdline.PARALLEL_COMMAND
}

function prepare_statements()
   -- We do not use prepared statements here, but oltp_common.sh expects this
   -- function to be defined
end

function event()
   local feature_count = 4428
   local table_name = "uscities"

   local column_name = "geom"
   local query_type = sysbench.rand.uniform(1,14)
   local geom_type = sysbench.rand.uniform(1,3)
   if geom_type == 1 then
      feature_count = 4428
      table_name = "uscities"
   elseif geom_type == 2 then
      feature_count = 7836
      table_name = "usroads"
   elseif geom_type == 3 then
      feature_count = 3219
      table_name = "uscounties"
   end

   -- id query
   local query_id = sysbench.rand.uniform(1,feature_count)
   local query_id_size = 100

   -- extent query
   local extent_xmin = -170.2773437497141344
   local extent_ymin = 17.7117061619100014
   local extent_xmax = -64.8812637328348387
   local extent_ymax = 70.6364212039755444
   local xrange = (extent_xmax - extent_xmin) / 20
   local yrange = (extent_ymax - extent_ymin) / 20

   local xmin = sysbench.rand.uniform_double(extent_xmin, extent_xmax)
   local ymin = sysbench.rand.uniform_double(extent_ymin,extent_ymax)
   local xmax = xmin + xrange
   local ymax = ymin + yrange

   if query_type == 1 then
      con:query(string.format("select st_astext(%s) from %s where gid>=%d AND gid < (%d + %d)",
                              column_name, table_name, query_id, query_id, query_id_size))
   elseif query_type == 2 then
      con:query(string.format("select st_asewkt(%s) from %s where gid>=%d AND gid < (%d + %d)",
                              column_name, table_name, query_id, query_id, query_id_size))
   elseif query_type == 3 then
      con:query(string.format("select ST_AsGeoJSON(%s) from %s where gid>=%d AND gid < (%d + %d)",
                              column_name, table_name, query_id, query_id, query_id_size))
   elseif query_type == 4 then
      con:query(string.format("select st_asbinary(%s) from %s where gid>=%d AND gid < (%d + %d)",
                              column_name, table_name, query_id, query_id, query_id_size))
   elseif query_type == 5 then
      con:query(string.format("select st_asewkb(%s) from %s where gid>=%d AND gid < (%d + %d)",
                              column_name, table_name, query_id, query_id, query_id_size))
   elseif query_type == 6 then
      con:query(string.format("select st_srid(%s) from %s where gid>=%d AND gid < (%d + %d)",
                              column_name, table_name, query_id, query_id, query_id_size))
   elseif query_type == 7 then
      con:query(string.format("select st_area(%s) from %s where gid>=%d AND gid < (%d + %d)",
                              column_name, table_name, query_id, query_id, query_id_size))
   elseif query_type == 8 then
      con:query(string.format("Select count(%s) from %s where %s && st_makebox2d(st_makepoint(%f, %f), st_makepoint(%f, %f))",
                              column_name, table_name, column_name, xmin, ymin, xmax, ymax))
   elseif query_type == 9 then
      con:query(string.format("Select st_intersects(%s, ST_GeomFromText('POLYGON((%f %f , %f  %f, %f %f, %f %f, %f %f ))', 0)) from %s where gid>=%d AND gid < (%d +%d)",
                              column_name, xmin, ymin, xmax, ymin, xmax, ymax, xmin, ymax, xmin, ymin,
                              table_name, query_id, query_id, query_id_size))
   elseif query_type == 10 then
      con:query(string.format("Select gid from %s where st_intersects(%s, ST_GeomFromText('POLYGON((%f %f , %f  %f, %f %f, %f %f, %f %f ))', 0)) ",
                              table_name, column_name, xmin, ymin, xmax, ymin, xmax, ymax, xmin, ymax, xmin, ymin ))   
   elseif query_type == 11 then
      con:query(string.format("Select st_within(%s, ST_GeomFromText('POLYGON((%f %f , %f  %f, %f %f, %f %f, %f %f ))', 0)) from %s where gid>=%d AND gid < (%d +%d)",
                              column_name,xmin, ymin, xmax, ymin, xmax, ymax, xmin, ymax, xmin, ymin,
                              table_name, query_id, query_id, query_id_size))
   elseif query_type == 12 then
      con:query(string.format("Select st_contains(ST_GeomFromText('POLYGON((%f %f , %f  %f, %f %f, %f %f, %f %f ))', 0),%s) from %s where gid>=%d AND gid < (%d +%d)",
                              xmin, ymin, xmax, ymin, xmax, ymax, xmin, ymax, xmin, ymin,column_name,
                              table_name, query_id, query_id, query_id_size))
   elseif query_type == 13 then
      con:query(string.format("Select st_intersection(%s, ST_GeomFromText('POLYGON((%f %f , %f  %f, %f %f, %f %f, %f %f ))', 0)) from %s where gid>=%d AND gid < (%d +%d)",
                              column_name, xmin, ymin, xmax, ymin, xmax, ymax, xmin, ymax, xmin, ymin,
                              table_name, query_id, query_id, query_id_size))
   elseif query_type == 14 then
      con:query(string.format("Select st_buffer(%s, 0.000001) from %s where gid>=%d AND gid < (%d +%d)",
                              column_name, table_name, query_id, query_id, query_id_size))                          
   end

   --throttle_event() --Calculate the time delay to achieve target TPS
end
