
-- 兼容新版本的sysbench
-- 原生的sysbench.rand.uniform发现不够均匀，采用lua的库函数
function get_uniform(s, e)
	return math.random(math.floor(s), math.floor(e))
end
sb_rand = sysbench.rand.default
sb_rand_str = sysbench.rand.string
sb_rand_uniform = get_uniform
sb_rand_gaussian = sysbench.rand.gaussian
sb_rand_special = sysbench.rand.special
sb_rand_pareto = sysbench.rand.pareto_new
get_global_seq = sysbench.rand.get_global_seq
set_global_seq_min = sysbench.rand.set_global_seq_min



if sysbench.cmdline.command == nil then
   error("Command is required. Supported commands: prepare, warmup, run, " ..
            "cleanup, help")
end

-- Command line options
sysbench.cmdline.options = {

    pk_random = {"1:random pk, 0: sequence", 0 },
    row_len =
      {"length of average pad column, max 3000, buyer ", 1500},
    table_size =
      {"Number of rows per table", 10000},
    oltp_range_size =
      {"Range size for range SELECT queries", 100},
    tables =
      {"Number of tables", 1},
    mysql_table_engine =
      {"Storage engine, if MySQL is used", "innodb"},
    thread_sleep = {"the us sleep after event done, will sleep random(thread_sleep/2, thread_sleep) us", 0 }
}


function init()
   assert(event ~= nil,
          "this script is meant to be included by other OLTP scripts and " ..
             "should not be called directly.")
end


function thread_init()
   drv = sysbench.sql.driver()
   con = drv:connect()
    -- This function is a 'callback' defined by individual benchmark scripts
   prepare_statements()
end


-- Prepare the dataset. This command supports parallel execution, i.e. will
-- benefit from executing with --threads > 1 as long as --tables > 1
-- 在prepare 命令时会执行该函数
function cmd_prepare()
   local drv = sysbench.sql.driver()
   local con = drv:connect()

   for i = sysbench.tid % sysbench.opt.threads + 1, sysbench.opt.tables,
   sysbench.opt.threads do
     create_table(drv, con, i)
   end
end

-- cleanup操作时的操作
function cleanup()
   local drv = sysbench.sql.driver()
   local con = drv:connect()

   for i = 1, sysbench.opt.tables do
      print(string.format("Dropping table 'buyer_resource_%d' ...", i))
      con:query("DROP TABLE IF EXISTS buyer_resource_" .. i )
   end
end

-- Implement parallel prepare and warmup commands, define 'prewarm' as an alias
-- for 'warmup'
-- 告诉sysbench在执行prepare等命令时并发调度
sysbench.cmdline.commands = {
   prepare = {cmd_prepare, sysbench.cmdline.PARALLEL_COMMAND},
}


function create_table(drv, con, i)

       local pk
       local query

       print("Creating table ".."buyer_resource_" .. i ..",".. "...")

        query = [[

        CREATE TABLE `buyer_resource_]] .. i .. [[` (
  `resource_id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '唯一ID号',
  `resource_provider` bigint(20) NOT NULL COMMENT '发放资源者（卖家or小二）',
  `buyer_id` bigint(20) NOT NULL COMMENT '买家ID',
  `resource_type` int(11) NOT NULL COMMENT '资源类型',
  `sub_resource_type` bigint(20) NOT NULL COMMENT '资源子类型',
  `status` tinyint(4) NOT NULL COMMENT '状态',
  `start_time` datetime NOT NULL COMMENT '活动有效开始时间',
  `end_time` datetime NOT NULL COMMENT '活动有效结束时间',
  `feature` varchar(4000) DEFAULT NULL COMMENT '扩展字段',
  `gmt_created` datetime NOT NULL COMMENT '创建时间',
  `gmt_modified` datetime NOT NULL COMMENT '最后修改时间',
  `source` int(11) DEFAULT NULL COMMENT '标识来源（淘券还是聚划算等）',
  `original_resource_id` bigint(20) DEFAULT NULL COMMENT '最初的资源id，若某个资源被转赠过，这个id表示第一个持有这个资源的resourceId',
  `seller_id` bigint(20) DEFAULT NULL COMMENT '发放资源的卖家id',
  `create_source` int(11) DEFAULT NULL COMMENT '标识创建来源:空为淘宝,1为聚划算,2为1688,等待废弃',
  `business_unit` tinyint(4) DEFAULT NULL COMMENT '业务对应的商业单元,不可为空',
  `feature2` varchar(4000) DEFAULT NULL COMMENT '扩展字段',
  `out_instance_type` tinyint(4) DEFAULT NULL COMMENT '外部关联实例id类型，,不可为空',
  `out_instance_id` varchar(128) DEFAULT NULL COMMENT '外部关联实例id（外部券实例id,订单id），,不可为空',
  `resource_code` bigint(20) DEFAULT NULL COMMENT '不可为空，资源code，和resource_type结合唯一确定一条记录。对于迁移过来的数据，该字段等于老表的资源id，否则这个字段的值与resource_id相等',
  `options` tinyint(4) DEFAULT NULL COMMENT '标记位',
  `consume_id` varchar(128) DEFAULT NULL COMMENT '消耗id，比如使用这张券的订单id',
  `consume_type` tinyint(4) DEFAULT NULL COMMENT '消耗类型',
  `parent_resource_code` bigint(20) DEFAULT NULL COMMENT '父亲资源code',
  `total_count` int(11) DEFAULT NULL COMMENT '总资源数量',
  `available_count` int(11) DEFAULT NULL COMMENT '可用资源数量',
  `in_use_count` int(11) DEFAULT NULL COMMENT '使用中资源数量',
  `used_count` int(11) DEFAULT NULL COMMENT '已经使用资源数量',
  `invalid_count` int(11) DEFAULT NULL COMMENT '失效资源数量',
  `version` bigint(20) unsigned DEFAULT '0' COMMENT '版本号',
  `group_id` bigint(20) unsigned DEFAULT '0' COMMENT 'groupId',
  `business_id` bigint(20) unsigned DEFAULT '0' COMMENT '业务ID',
  `rule` varchar(4000) DEFAULT NULL COMMENT '模版维度的规则',
  `market_place` varchar(32) NOT NULL DEFAULT 'TAO' COMMENT '市场：ICBU、AE、SJ等,默认为TAO，代表淘系',
  PRIMARY KEY (`resource_id`),
  UNIQUE KEY `uk_transfer_use` (`resource_type`,`resource_code`),
  KEY `idx_buyer_resource_buyerid` (`buyer_id`,`end_time`,`resource_type`,`status`,`start_time`,`sub_resource_type`,`resource_provider`),
  KEY `idx_buyer_id_seller_id_status_end_time` (`buyer_id`,`seller_id`,`status`,`end_time`),
  KEY `idx_buyer_id_status_resource_provider_resource_type` (`buyer_id`,`status`,`resource_provider`,`resource_type`),
  KEY `idx_gmt_modified` (`gmt_modified`),
  KEY `idx_bid_respri_restype_cid` (`buyer_id`,`resource_provider`,`resource_type`,`consume_id`),
  KEY `idx_idempt` (`buyer_id`,`out_instance_id`,`out_instance_type`),
  KEY `idx_zhisheng` (`buyer_id`,`status`,`seller_id`)
) ENGINE=]].. sysbench.opt.mysql_table_engine ..[[  AUTO_INCREMENT=324250940833 DEFAULT CHARSET=utf8mb4 COMMENT='买家持有资源表';

        ]]

        con:query(query)
        con:bulk_insert_init("INSERT INTO buyer_resource_" .. i .. " (resource_code,gmt_modified,resource_id,out_instance_type,create_source,consume_type,gmt_created,out_instance_id,market_place,source,feature,consume_id,business_unit,version,status,sub_resource_type,start_time,business_id,original_resource_id,resource_provider,seller_id,end_time,group_id,options,resource_type,buyer_id) VALUES")

        -- bulk insert
        for pk = 1, sysbench.opt.table_size do
            con:bulk_insert_next("("..pk..",".."now(),"..pk..","..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 10)..",".."now(),".."'"..sb_rand_str(string.rep('@', 10)).."',".."'"..sb_rand_str(string.rep('@', 3)).."',"..sb_rand(1, 10)..",".."'"..sb_rand_str(string.rep('@', 1200)).."',".."'"..sb_rand_str(string.rep('@', 15)).."',"..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 1000000000)..",".."now(),"..sb_rand(1, 10)..","..sb_rand(1, 100000000000)..","..sb_rand(1, 1000000000)..","..sb_rand(1, 1000000000)..",".."now(),"..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 1000000000)..")")
        end
        --commit
        con:bulk_insert_done()


end


function get_pkid(max_num)
    -- 用于根据线程号区分取值区间，不同线程之间的事务不会触发行锁
    -- 获取当前线程号
    local _start
    local _end
    _start =  max_num *  (sysbench.tid/sysbench.opt.threads)
    _end = max_num *  ((sysbench.tid + 1 )/sysbench.opt.threads)
    return sb_rand_uniform(_start, _end)
end



function insert()

    local table
    local sql
    local pk
    local uk
    pk = 0
    uk = get_pkid(100000000000000000)
    table = "buyer_resource".."_".. sb_rand_uniform(1, sysbench.opt.tables)
    -- "now()," 替换成 "'"..time.."',"
    sql = "INSERT ignore INTO " .. table .. " (resource_code,gmt_modified,resource_id,out_instance_type,create_source,consume_type,gmt_created,out_instance_id,market_place,source,feature,consume_id,business_unit,version,status,sub_resource_type,start_time,business_id,original_resource_id,resource_provider,seller_id,end_time,group_id,options,resource_type,buyer_id) VALUES"
            .."("..uk..",".."'"..time.."',"..pk..","..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 10)..",".."'"..time.."',".."'"..sb_rand_str(string.rep('@', 10)).."',".."'"..sb_rand_str(string.rep('@', 3)).."',"..sb_rand(1, 10)..",".."'"..sb_rand_str(string.rep('@', 1200)).."',".."'"..sb_rand_str(string.rep('@', 15)).."',"..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 1000000000)..",".."'"..time.."',"..sb_rand(1, 10)..","..sb_rand(1, 100000000000)..","..sb_rand(1, 1000000000)..","..sb_rand(1, 1000000000)..",".."'"..time.."',"..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 10)..","..sb_rand(1, 1000000000)..")"
    con:query(sql)
end



function update()

    local table
    local sql
    table = "buyer_resource".."_".. sb_rand_uniform(1, sysbench.opt.tables)
    sql = "update ".. table .." SET STATUS = "..sb_rand(1, 10)..", gmt_modified = '"..time.."', version = version + 1, consume_id = ".."'"..sb_rand_str(string.rep('@', 40)).."',"
            .."consume_type = "..sb_rand(1, 10) .. " where resource_id=" ..get_pkid(sysbench.opt.table_size)
    con:query(sql)

end


function point_select()
    local table = "buyer_resource".."_".. sb_rand_uniform(1, sysbench.opt.tables)
    local stmt= "select buyer_id,status,resource_provider,resource_type from "..table.." where resource_id="..get_pkid(sysbench.opt.table_size)
    con:query(stmt)
end

function range_select()

    local table = "buyer_resource".."_".. sb_rand_uniform(1, sysbench.opt.tables)
    local stmt = " select * from "..table.."  where resource_id <="..get_pkid(sysbench.opt.table_size).." limit 100"
    con:query(stmt)
end


function query()
    local table = "buyer_resource".."_".. sb_rand_uniform(1, sysbench.opt.tables)
    local stmt = "select count(1) from "..table.." where resource_id=" ..get_pkid(sysbench.opt.table_size).." and status="..sb_rand(1, 10).." "
    con:query(stmt)
end
