#!/usr/bin/env sysbench
-- Copyright (C) 2006-2017 Alexey Kopytov <akopytov@gmail.com>

-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

-- ----------------------------------------------------------------------
-- Merge OLTP benchmark for PolarDB-O
-- ----------------------------------------------------------------------

require("oltp_common")

sysbench.cmdline.commands.prepare = {
   function ()
      if (not sysbench.opt.auto_inc) then
         -- Create empty tables on prepare when --auto-inc is off, since IDs
         -- generated on prepare may collide later with values generated by
         -- sysbench.rand.unique()
         sysbench.opt.table_size=0
      end

      cmd_prepare()
   end,
   sysbench.cmdline.PARALLEL_COMMAND
}

function prepare_statements()
   -- We do not use prepared statements here, but oltp_common.sh expects this
   -- function to be defined
end

function event()
   local table_name = "sbtest" .. sysbench.rand.uniform(1, sysbench.opt.tables)
   local table_name2 = "sbtest" .. sysbench.rand.uniform(1, sysbench.opt.tables)
   local merge_type = sysbench.rand.uniform(1,3)
   while (table_name == table_name2)
   do
      table_name2 = "sbtest" .. sysbench.rand.uniform(1, sysbench.opt.tables)
   end
   local k_val = sysbench.rand.default(1, sysbench.opt.table_size)
   local k_val2 = sysbench.rand.default(k_val, sysbench.opt.table_size)

   if merge_type == 1 then
      con:query(string.format("MERGE INTO %s t USING (SELECT * FROM %s WHERE k BETWEEN %d AND %d) AS s ON s.id = t.id " ..
                                 "WHEN MATCHED THEN DELETE WHEN NOT MATCHED THEN INSERT VALUES(s.id, s.k, 'Not Match1', 'Inserted1')",
                              table_name, table_name2, k_val, k_val2))
   elseif merge_type == 2 then
      con:query(string.format("MERGE INTO %s t USING (SELECT * FROM %s WHERE k BETWEEN %d AND %d) AS s ON s.id = t.id " ..
                                 "WHEN MATCHED THEN UPDATE SET pad = 'Updated2' WHEN NOT MATCHED THEN INSERT VALUES(s.id, s.k, 'Not Match2', 'Inserted2')",
                              table_name, table_name2, k_val, k_val2))
   elseif merge_type == 3 then
      con:query(string.format("MERGE INTO %s t USING (SELECT * FROM %s WHERE k BETWEEN %d AND %d) AS s ON s.id = t.id " ..
                                 "WHEN MATCHED AND S.ID > 500000 THEN UPDATE SET pad = 'Updated3' " ..
                                 "WHEN MATCHED THEN DELETE " ..
                                 "WHEN NOT MATCHED THEN INSERT VALUES(s.id, s.k, 'Not Match3', 'Inserted3')",
                              table_name, table_name2, k_val, k_val2))
   end

   throttle_event() --Calculate the time delay to achieve target TPS
end
