#!/usr/bin/env sysbench
-- Copyright (C) 2006-2017 Alexey Kopytov <akopytov@gmail.com>

-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

-- ----------------------------------------------------------------------
-- Test for PIVOT for PolarDB-O
-- It uses sysbench TPCC tables
-- ----------------------------------------------------------------------

require("oltp_common")

sysbench.cmdline.commands.prepare = {
   function ()
      if (not sysbench.opt.auto_inc) then
         -- Create empty tables on prepare when --auto-inc is off, since IDs
         -- generated on prepare may collide later with values generated by
         -- sysbench.rand.unique()
         sysbench.opt.table_size=0
      end

      cmd_prepare()
   end,
   sysbench.cmdline.PARALLEL_COMMAND
}

function prepare_statements()
   -- We do not use prepared statements here, but oltp_common.sh expects this
   -- function to be defined
end

function event()
   -- Define tables
   local table_name = "new_orders" .. sysbench.rand.uniform(1, sysbench.opt.pivot_tables)
   local table_name2 = "orders" .. sysbench.rand.uniform(1, sysbench.opt.pivot_tables)
   local table_name3 = "history" .. sysbench.rand.uniform(1, sysbench.opt.pivot_tables)
   local table_name4 = "order_line" .. sysbench.rand.uniform(1, sysbench.opt.pivot_tables)
   local query_type = sysbench.rand.uniform(1,5)

   -- values used in the in clause
   local k_val = sysbench.rand.default(1, sysbench.opt.pivot_scale)
   local k_val2 = sysbench.rand.default(1, sysbench.opt.pivot_scale)

   if query_type == 1 then
      con:query(string.format("select * from %s pivot(sum(no_d_id) as sum_d_id for no_w_id in (%d,%d)) ",
                              table_name, k_val, k_val2))
   elseif query_type == 2 then
      con:query(string.format("select * from (select o_w_id, o_carrier_id, o_ol_cnt from %s) pivot(sum(o_ol_cnt) " ..
                                 "as sum_cnt for o_carrier_id in (%d,%d))",
                              table_name2, k_val, k_val2))
   elseif query_type == 3 then
      con:query(string.format("select * from (select h_c_id, h_amount,h_c_d_id from %s) pivot(sum(h_amount) " ..
                                 "as sum_amount for h_c_d_id in (%d,%d))",
                              table_name3, k_val, k_val2))
   elseif query_type == 4 then
      con:query(string.format("select * from (select h_c_id, h_amount,h_c_d_id,h_c_w_id from %s) pivot(sum(h_amount) sum_amount, " ..
                                 "max(h_c_w_id) max for h_c_d_id in (%d,%d))",
                              table_name3, k_val, k_val2))
   elseif query_type == 5 then
      con:query(string.format("select * from (select ol_number, ol_i_id, ol_supply_w_id, ol_quantity from %s) pivot(sum(ol_quantity), " ..
                                 "max(ol_i_id) as max_id for ol_number in (%d,%d))",
                              table_name4, k_val, k_val2))
   end

   throttle_event() --Calculate the time delay to achieve target TPS
end
