-- -------------------------------------------------------------------------- --
-- Postgres Cophy benchmark: do copy from table and copy to table concurrently in --threads
-- threads with each thread copy from/to its own table. The number of copy
-- executed by each thread is controlled by --table_size.
-- -------------------------------------------------------------------------- --
-- Command line options
sysbench.cmdline.options = {
    table_size =
        {"Number of rows per copy", 100}
}

function thread_init()
   drv = sysbench.sql.driver()
   con = drv:connect()
end

function prepare()
    local i

    print("sysbench.opt.table_size is " .. sysbench.opt.table_size)
    print("sysbench.opt.threads is " .. sysbench.opt.threads)
    
    local drv = sysbench.sql.driver()
    local con = drv:connect()
 
    for i = 1, sysbench.opt.threads do
       print("Creating table 'sbtest" .. i .. "'...")
       con:query(string.format([[
         CREATE TABLE IF NOT EXISTS sbtest%d (
           id INTEGER NOT NULL,
           k INTEGER DEFAULT '0' NOT NULL)]], i))
    end
    for i = 0, sysbench.opt.table_size do
        if (i == 0) then
            con:bulk_insert_init("INSERT INTO sbtest1 VALUES")
        end
      
        i = i + 1
      
        con:bulk_insert_next("(" .. i .. "," .. i .. ")")
    end
    con:bulk_insert_done()
    con:query([[COPY sbtest1  TO '/tmp/copy_tmp_data.csv' WITH csv]])
    con:query([[truncate table sbtest1]])
end


function event()
    con:query([[COPY sbtest]] .. sysbench.tid+1 .. [[  FROM '/tmp/copy_tmp_data.csv' WITH csv]])
end

function thread_done()
    con:disconnect()
end

function cleanup()
    local i
 
    local drv = sysbench.sql.driver()
    local con = drv:connect()

    os.remove("/tmp/copy_tmp_data.csv")

    for i = 1, sysbench.opt.threads do
       print("Dropping table 'sbtest" .. i .. "'...")
       con:query("DROP TABLE IF EXISTS sbtest" .. i )
    end
end