--[[
****************************************************************************
Usage:

run:

nohup sysbench /root/sysbench/src/lua/pg_random_db_schema.lua --report-interval=1
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --time=0
--threads=50 --pgsql-ignore-errors=42710  run > random_db.txt 2>&1 &

****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps =
       {"Desiried transaction throughput (TPS)",-1}

}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function thread_init(thread_id)
    local i, j, k, n_fks
    local stmt
    local child_keys, parent_keys

    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end
end

-- The main event loop
function event()
    cur_table = tables[sysbench.rand.uniform(1,n_table)]
    -- To add drop frequency, added more drop calls
    local max_op =  6

    local op_type = sysbench.rand.uniform(1,max_op)
    if op_type == 1 then
        op = "create_db"
    elseif op_type == 2 then
        op = "drop_db"
    elseif op_type == 3 then
        op = "drop_db"
    elseif op_type == 4 then
        op = "create_schema"
    elseif op_type == 5 then
        op = "drop_schema"
    elseif op_type == 6 then
        op = "drop_schema"
    end
    _G[op]()
    throttle_event()
end

function create_db()
    print("create database ...")
    stmt = ([[select datname from pg_database
              where datname like 'rand_db_%%']])
    rs = con:query(stmt)
    if rs.nrows >=500 then
        print("Already has maximum of 500 random databases")
        return
    end

    db_no = sysbench.rand.uniform(1,99999999)
    db_name = "rand_db_"..db_no

    stmt = ([[create database %s]]):format(db_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function drop_db()
    print("Drop database ... ")
    stmt = ([[select datname from pg_database
              where datname like 'rand_db_%%']])
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("No database to drop")
        return
    end

    db_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, db_no do
        row = rs:fetch_row()
        db_name = tostring(row[1])
    end

    stmt = ([[drop database if exists %s]]):format(db_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function create_schema()
    print("create schema ...")
    stmt = ([[select schema_name from information_schema.schemata
              where schema_name like 'rand_sch_%%']])
    rs = con:query(stmt)
    if rs.nrows >=500 then
        print("Already has maximum of 500 random schemas")
        return
    end

    sch_no = sysbench.rand.uniform(1,99999999)
    sch_name = "rand_sch_"..sch_no

    stmt = ([[create schema if not exists %s]]):format(sch_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function drop_schema()
    print("Drop database ... ")
    stmt = ([[select schema_name from information_schema.schemata
              where schema_name like 'rand_sch_%%']])
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("No schema to drop")
        return
    end

    sch_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, sch_no do
        row = rs:fetch_row()
        sch_name = tostring(row[1])
    end

    stmt = ([[drop schema if exists %s]]):format(sch_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end
