--[[
****************************************************************************
Usage:

setup:

sysbench /root/sysbench/src/lua/oltp_insert.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --tables=20 cleanup

sysbench /root/sysbench/src/lua/oltp_insert.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --rand-type=uniform --tables=20
--table-size=100000 prepare

run:

nohup sysbench /root/sysbench/src/lua/pg_random_ddl.lua --report-interval=1
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --time=0
--threads=50 run > random_ddl.txt 2>&1 &

****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps =
       {"Desiried transaction throughput (TPS)",-1}

}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function thread_init(thread_id)
    local i, j, k, n_fks
    local stmt
    local child_keys, parent_keys

    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    con1 = drv:connect()
    con2 = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end

    -- Enum current schema tables
    rs = con:query("select table_name from information_schema.tables " ..
                   "where table_type = 'BASE TABLE' and table_name like 'sbtest%'")
    n_table = rs.nrows
    for i = 1, n_table do
        row = rs:fetch_row()
        tables[i] = tostring(row[1])
    end

end

function prepare()

    local drv = sysbench.sql.driver()
    local con = drv:connect()

    print("Create extensions btree_gin")
    con:query("create extension if not exists btree_gin")

    print("Create extensions btree_gist")
    con:query("create extension if not exists btree_gist")

end

-- The main event loop
function event()
    cur_table = tables[sysbench.rand.uniform(1,n_table)]

    local max_op =  7

    local op_type = sysbench.rand.uniform(1,max_op)
    if op_type == 1 then
      op = "vacuum_full"
    elseif op_type == 2 then
        op = "add_column"
    elseif op_type == 3 then
        op = "drop_column"
    elseif op_type == 4 then
        op = "add_index"
    elseif op_type == 5 then
        op = "drop_index"
    elseif op_type == 6 then
        op = "change_column"
    elseif op_type == 7 then
        op = "change_storage_att"
    end
    _G[op](cur_table)
    throttle_event()
end

function vacuum_full(table_name)
    print("vacuum full "..table_name)
    ddl_con:query("vacuum full "..table_name)
end

function add_column(table_name)
    print("Add column to table "..table_name)
    stmt = ([[select a.attname as name from pg_class as c,pg_attribute as a
               where c.relname = '%s' and a.attrelid = c.oid and a.attnum>0 and
               a.attname like 'rand_ddl_%%']]):format(table_name)
    rs = con:query(stmt)
    if rs.nrows >=200 then
        print("Table already has maximum of 200 random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,99999999)
    col_name = "rand_ddl_"..col_no

    op = sysbench.rand.uniform(1,6)
    if op == 1 then
        col_def = 'int default 0 not null'
    elseif op == 2 then
        col_def = "varchar(100) default 'this is a long long test string'"
    elseif op == 3 then
        col_def = 'decimal(20,2) default 2.2'
    elseif op == 4 then
        col_def = "timestamp default '2016-11-16 08:18:18'"
    elseif op == 5 then
        col_def = "text default 'this is a text'"
    else
        col_def = 'bytea'
    end

    stmt = ([[alter table %s add column if not exists %s %s]]):format(table_name, col_name, col_def)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function drop_column(table_name)
    print("Drop column from table "..table_name)
    stmt = ([[select a.attname as name from pg_class as c,pg_attribute as a
               where c.relname = '%s' and a.attrelid = c.oid and a.attnum>0 and
               a.attname like 'rand_ddl_%%' and
               cast( nullif( substring(a.attname,10,length(a.attname)), '') as int) between 1 and 55555555]]):format(table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[alter table %s drop column if exists %s]]):format(table_name, col_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function add_index(table_name)
    print("Add index to table "..table_name)
    stmt = ([[select a.attname as name from pg_class as c,pg_attribute as a
               where c.relname = '%s' and a.attrelid = c.oid and a.attnum>0 and
               a.attname like 'rand_ddl_%%' and
               cast( nullif( substring(a.attname,10,length(a.attname)), '') as int) between 55555556 and 99999999]]):format(table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    idx_no = sysbench.rand.uniform(1,99999999)
    idx_name = "rand_idx_"..idx_no
    op_idx_type = 4

    if op_idx_type == 1 then
        idx_def = 'gin'
    elseif op_idx_type == 2 then
        idx_def = 'gist'
    elseif op_idx_type == 3 then
        idx_def = 'hash'
    elseif op_idx_type == 4 then
        idx_def = 'brin'
    else
        idx_def = 'btree'
    end

    stmt = ([[create index concurrently if not exists %s on %s using %s (%s) ]]):format(idx_name,table_name,idx_def,col_name)
    print(stmt)
    rs = ddl_con:query(stmt)

end

function drop_index(table_name)
    print("Drop index from table "..table_name)
    stmt = ([[select a.attname as name from pg_class as c,pg_attribute as a
               where c.relname = '%s' and a.attrelid = c.oid and a.attnum>0 and
               a.attname like 'rand_ddl_%%']]):format(table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[select indexname from pg_indexes where tablename= '%s' and indexname like 'rand_idx_%%']]):format(table_name)
    rs = con:query(stmt)

    if rs.nrows == 0 then
        print(([[Table %s has no index to drop]]):format(table_name))
        return
    end

    row = rs:fetch_row()
    idx_name = tostring(row[1])
    stmt = ([[drop index concurrently IF EXISTS %s]]):format(idx_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end


function change_storage_att(table_name)
    local i, op, stmt
    -- alter table change STORAGE attribute
    stmt = ([[select a.attname from pg_class as c,pg_attribute as a
              where c.relname = '%s' and a.attrelid = c.oid and a.attnum>0 and
              a.attname like 'rand_ddl_%%' and format_type(a.atttypid,a.atttypmod) not in ('xml','integer') and
              format_type(a.atttypid,a.atttypmod) not like 'timestamp%%' and
              cast( nullif( substring(a.attname,10,length(a.attname)), '') as int) between 55555556 and 99999999]]):format(table_name)
    rs = con:query(stmt)

    if rs.nrows == 0 then
        print(([[Table %s has no suitable columns to alter]]):format(table_name))
        return
    end

    row = rs:fetch_row()
    chg_col_name = tostring(row[1])

    op_storage_type = sysbench.rand.uniform(1,3)

    if op_storage_type == 1 then
        storage_def = 'EXTERNAL'
    elseif op_storage_type == 2 then
        storage_def = 'EXTENDED'
    else
        storage_def = 'MAIN'
    end

    stmt = ([[alter table if exists %s alter column %s set storage %s]]):format(table_name,chg_col_name,storage_def)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function change_column(table_name)
    local i
    local stmt
    print("Changing coloumn data type for "..table_name)
    stmt = ([[select t_new.comment,substring(t_new.type,0,t_new.begin) as type,t_new.col_name, t_new.notnull,
            substring(t_new.type,t_new.begin+1,t_new.end - t_new.begin -1) as length
            from ( SELECT col_description(a.attrelid,a.attnum) as comment,
                   format_type(a.atttypid,a.atttypmod) as type,
                   a.attname as col_name,
                   a.attnotnull as notnull,
                   position('(' in format_type(a.atttypid,a.atttypmod)) as begin,
                   position(')' in format_type(a.atttypid,a.atttypmod)) as end
                   FROM pg_class as c, pg_attribute as a where c.relname = '%s' and
                   a.attrelid = c.oid and a.attnum>0 and
                   cast( nullif( substring(a.attname,10,length(a.attname)), '') as int) between 55555556 and 99999999) as t_new
                   where type like 'character%%']]):format(table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no char/varchar type")
        return
    end

    pos = sysbench.rand.uniform(1,rs.nrows)
    for i = 1, pos do
        row = rs:fetch_row()
    end
    col_name = tostring(row[3])
    is_not_null = tostring(row[4])
    data_type = tostring(row[2])
    len = tonumber(row[5])
    comment = tostring(row[1])

    if is_not_null == 't' then
        null_clause = 'NOT NULL'
    else
        null_clause = 'NULL'
    end

    if (len >=2000) then
        len = len - 100
    else
        len = len + 100
    end
    stmt = ([[alter table %s alter column %s type %s(%d)]]):format(
        table_name, col_name, data_type, len)
    print(stmt)
    ddl_con:query(stmt)
end

function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end
