--[[
****************************************************************************
Usage:

setup:

sysbench /root/sysbench/src/lua/oltp_insert.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --tables=20 cleanup

sysbench /root/sysbench/src/lua/oltp_insert.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --rand-type=uniform --tables=20
--table-size=100000 prepare

create trigger function:


run:

nohup sysbench /root/sysbench/src/lua/pg_random_trigger.lua --report-interval=1
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --time=0
--threads=50 --pgsql-ignore-errors=42710  run > random_ddl.txt 2>&1 &

nohup sysbench /root/sysbench/src/lua/oltp_delete.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --rand-type=uniform --tables=20
run > random_ddl_oltp_delete.txt 2>&1 &

nohup sysbench /root/sysbench/src/lua/oltp_delete.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --rand-type=uniform --tables=20
run > random_ddl_read_write.txt 2>&1 &

****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps =
       {"Desiried transaction throughput (TPS)",-1}

}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function thread_init(thread_id)
    local i, j, k, n_fks
    local stmt
    local child_keys, parent_keys

    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    con1 = drv:connect()
    con2 = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end

    -- Enum current schema tables
    rs = con:query("select table_name from information_schema.tables " ..
                   "where table_type = 'BASE TABLE' and table_name like 'sbtest%'")
    n_table = rs.nrows
    for i = 1, n_table do
        row = rs:fetch_row()
        tables[i] = tostring(row[1])
    end
end

function prepare()

    local drv = sysbench.sql.driver()
    local con = drv:connect()

    print("Create result table for trigger")
    con:query("CREATE TABLE IF NOT EXISTS trigger_test(op char(10), time timestamp)")

    print("Create trigger function")
    con:query("CREATE OR REPLACE FUNCTION log_trigger() RETURNS TRIGGER AS $$ " .. 
              "   BEGIN" ..
              "    IF (TG_OP = 'DELETE') THEN" ..
              "       INSERT INTO trigger_test VALUES('D', now());" ..
              "       RETURN OLD;" ..
              "    ELSIF (TG_OP = 'INSERT') THEN" ..
              "       INSERT INTO trigger_test VALUES('I', now());" ..
              "       RETURN NEW;" ..
              "    ELSIF (TG_OP = 'UPDATE') THEN" ..
              "       INSERT INTO trigger_test VALUES('U', now());" ..
              "       RETURN NEW;" ..
              "       END IF;" ..
              "    END;" ..
              "$$ LANGUAGE plpgsql")
end

-- The main event loop
function event()
    cur_table = tables[sysbench.rand.uniform(1,n_table)]

    local max_op =  2

    local op_type = sysbench.rand.uniform(1,max_op)
    if op_type == 1 then
        op = "create_trigger"
    elseif op_type == 2 then
        op = "drop_trigger"
    elseif op_type == 3 then
        op = "drop_column"
    elseif op_type == 4 then
        op = "add_index"
    elseif op_type == 5 then
        op = "drop_index"
    elseif op_type == 6 then
        op = "change_column"
    elseif op_type == 7 then
        op = "change_storage_att"
    end
    _G[op](cur_table)
    throttle_event()
end

function create_trigger(table_name)
    print("create trigger on "..table_name)
    stmt = ([[select tgname from pg_trigger,pg_class
              where tgname like 'rand_trg_%%' and
              pg_trigger.tgrelid=pg_class.oid and
              pg_class.relname ='%s']]):format(table_name)
    rs = con:query(stmt)
    if rs.nrows >=100 then
        print("Table already has maximum of 100 random triggers")
        return
    end

    trg_no = sysbench.rand.uniform(1,99999999)
    trg_name = "rand_trg_"..trg_no

    op = sysbench.rand.uniform(1,6)
    if op == 1 then
        trg_def = 'before update'
    elseif op == 2 then
        trg_def = 'before delete'
    elseif op == 3 then
        trg_def = 'before insert'
    elseif op == 4 then
        trg_def = 'after update'
    elseif op == 5 then
        trg_def = 'after delete'
    else
        trg_def = 'after insert'
    end

    op2 = sysbench.rand.uniform(1,2)
    if op2 == 1 then
        trg_def2 = 'for each row'
    else
        trg_def2 = 'for each statement'
    end

    stmt = ([[create trigger %s %s on %s
              %s execute function log_trigger()]]):format(trg_name, trg_def, table_name, trg_def2)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function drop_trigger(table_name)
    print("Drop trigger from table "..table_name)
    stmt = ([[select tgname from pg_trigger,pg_class
              where tgname like 'rand_trg_%%' and
              pg_trigger.tgrelid=pg_class.oid and
              pg_class.relname ='%s']]):format(table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no triggers to drop")
        return
    end

    trg_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, trg_no do
        row = rs:fetch_row()
        trg_name = tostring(row[1])
    end

    stmt = ([[drop trigger if exists %s on %s cascade]]):format(trg_name, table_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end
