--[[
****************************************************************************
Usage:

setup:

sysbench /root/sysbench/src/lua/oltp_insert.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --tables=20 cleanup

sysbench /root/sysbench/src/lua/oltp_insert.lua --report-interval=10 --threads=20
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --rand-type=uniform --tables=20
--table-size=100000 prepare

run:

nohup sysbench /root/sysbench/src/lua/pg_random_ddl.lua --report-interval=1
--db-driver=pgsql --pgsql-host=11.167.232.100 --pgsql-db=sbtest --pgsql-port=5432
--pgsql-user=sbtest --pgsql-password=password --time=0
--threads=50 run > random_ddl.txt 2>&1 &

****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps =
       {"Desiried transaction throughput (TPS)",-1}

}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function thread_init(thread_id)
    local i
    local stmt

    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end

    -- Enum current schema tables
    rs = con:query("select table_name from information_schema.tables " ..
                   "where table_type = 'BASE TABLE' and table_name like 'sbtest%'")
    n_table = rs.nrows
    for i = 1, n_table do
        row = rs:fetch_row()
        tables[i] = tostring(row[1])
    end

end

function prepare()

    local drv = sysbench.sql.driver()
    local con = drv:connect()
    print("Running prepare")
end

-- The main event loop
function event()
    cur_table = tables[sysbench.rand.uniform(1,n_table)]

    local max_op = 10

    local op_type = sysbench.rand.uniform(1,max_op)
    if op_type == 1 then
      op = "vacuum_full"
    elseif op_type == 2 then
        op = "vacuum"
    elseif op_type == 3 then
        op = "vacuum"
    elseif op_type == 4 then
        op = "vacuum"
    elseif op_type == 5 then
        op = "vacuum"
    elseif op_type == 6 then
        op = "vacuum"
    elseif op_type == 7 then
        op = "vacuum_freeze"
    elseif op_type == 8 then
        op = "vacuum_freeze"
    elseif op_type == 9 then
        op = "vacuum_analyze"
    elseif op_type == 10 then
        op = "vacuum_analyze"
    end
    _G[op](cur_table)
    throttle_event()
end

function vacuum_full(table_name)
    print("vacuum full "..table_name)
    ddl_con:query("vacuum full "..table_name)
end

function vacuum(table_name)
    print("vacuum "..table_name)
    ddl_con:query("vacuum "..table_name)
end

function vacuum_freeze(table_name)
    print("vacuum freeze "..table_name)
    ddl_con:query("vacuum freeze "..table_name)
end

function vacuum_analyze(table_name)
    print("vacuum analyze "..table_name)
    ddl_con:query("vacuum analyze "..table_name)
end


function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end
