--[[
****************************************************************************
Input parameters:

--simsim-databases = N: the number of databases to create/use (1 by default)
****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps = 
       {"Desiried transaction throughput (TPS)",-1}
 
}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function log_with_ts(str)
    print(([[%s(%s) %s]]):format(os.date("%Y-%m-%dT%T"), os.time(), str))
end

function thread_init(thread_id)
    local i, j, k, n_fks
    local stmt
    local child_keys, parent_keys
    if sysbench.opt.threads > 1 then
        error("Only one thread is allowed.")
    end
       
    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    con1 = drv:connect()
    con2 = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end

    -- Enum current schema tables
    rs = con:query("select table_name from information_schema.tables " ..
                   "where table_schema = '"..sysbench.opt.mysql_db.."'" ..
                   "and table_type = 'BASE TABLE' and table_name not like '%_ostr'")
    n_table = rs.nrows
    for i = 1, n_table do
        row = rs:fetch_row()
        tables[i] = tostring(row[1])
    end

end


-- The main event loop
function event()
    con:query("USE " .. sysbench.opt.mysql_db)
    cur_table = tables[sysbench.rand.uniform(1,n_table)]

    local max_op =  8

    local op_type = sysbench.rand.uniform(1,max_op)
    if op_type == 1 then
      op = "optimize_table"
    elseif op_type == 2 then
        op = "add_column"
    elseif op_type == 3 then
        op = "drop_column"
    elseif op_type == 4 then
        op = "add_index"
    elseif op_type == 5 then
        op = "drop_index"
    elseif op_type == 6 then
        op = "change_column"
    elseif op_type == 7 then
        op = "change_row_format"
    else
        op = "create_temp_table"
    end
    _G[op](cur_table)
    throttle_event()
end           
   
function optimize_table(table_name)
    print("Optimize table "..table_name)
    stmt = "optimize table "..table_name
    log_with_ts(stmt)
    ddl_con:query(stmt)
end

function create_temp_table(table_name)
    print("create temporary table tmp_"..table_name)

    stmt="select @@autocommit auto_commit"
    rs = con:query(stmt)
    row = rs:fetch_row()
    auto_commit = tonumber(row[1])

    rs = con:query("set autocommit=1")

    local tbl_name = "tmp_"..table_name..sysbench.rand.uniform(1,100000)
    pre_stmt="drop temporary table if exists "..tbl_name
    con:query(pre_stmt)
    stmt = "create temporary table if not exists "..tbl_name.." (col1 int,col2 int,col3 int)"
    log_with_ts(stmt)
    ddl_con:query(stmt)

    rs = con:query("set autocommit="..auto_commit)

end

function add_column(table_name)
    print("Add column to table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and 
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows >=50 then 
        print("Table already has maximum of 50 random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,99999999)
    col_name = "rand_ddl_"..col_no

    op = sysbench.rand.uniform(1,6)
    if op == 1 then
        col_def = 'int default 0 not null'
    elseif op == 2 then
        col_def = "varchar(100) default 'this is a long long test string'"
    elseif op == 3 then
        col_def = 'decimal(20,2) default 2.2'
    elseif op == 4 then
        col_def = 'double default 1'
    elseif op == 5 then
        col_def = "datetime default '2012-12-12 08:12:23'"
    else
        col_def = 'binary(20)'
    end

    stmt = ([[alter table %s.%s add column %s %s]]):format(
                   sysbench.opt.mysql_db, table_name, col_name, col_def)
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)
end

function drop_column(table_name)
    print("Drop column from table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and 
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[alter table %s.%s drop column %s]]):format(
                   sysbench.opt.mysql_db, table_name, col_name)
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)
end

function add_index(table_name)
    print("Add index to table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and 
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[SELECT index_name FROM information_schema.statistics 
               WHERE table_schema = '%s' and table_name = '%s' and 
               index_name <> 'PRIMARY' and index_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name, col_name)
    rs = con:query(stmt)

    if rs.nrows ~= 0 then 
        print(([[Column %s alrady has index]]):format(col_name))
        return
    end

    stmt = ([[alter table %s.%s add index %s ( %s )]]):format(
                   sysbench.opt.mysql_db, table_name, col_name, col_name)
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)
end

function drop_index(table_name)
    print("Drop index from table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and 
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[SELECT index_name FROM information_schema.statistics 
               WHERE table_schema = '%s' and table_name = '%s' and 
               index_name <> 'PRIMARY' and index_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name, col_name)
    rs = con:query(stmt)

    if rs.nrows == 0 then 
        print(([[Column %s has no index]]):format(col_name))
        return
    end
    
    row = rs:fetch_row()
    col_name = tostring(row[1])
    stmt = ([[alter table %s.%s drop index %s ]]):format(
                   sysbench.opt.mysql_db, table_name, col_name, col_name)
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)
end


function change_row_format(table_name)
    local i, r_fmt, fl, op, stmt
    -- If table is partitioned by KEY, we can't drop the PK.
    stmt = ([[select distinct ROW_FORMAT from information_schema.tables
               where table_schema = '%s' and table_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    r_fmt = tostring(rs:fetch_row()[1])
    if r_fmt == 'Redundant' then 
        fl = 1
    elseif r_fmt == 'Compact' then 
        fl = 2
    elseif r_fmt == 'Compressed' or r_fmt == 'Dynamic' then 
        fl = 3
    end

    op = sysbench.rand.uniform(fl,4)
    if op == 1 then
        r_fmt = 'Redundant'
    elseif op == 2 then
        r_fmt = 'Compact'
    elseif op == 3 then
        r_fmt = 'Compressed'
    else
        r_fmt = 'Dynamic'
    end

    stmt = ([[alter table %s.%s row_format = %s]]):format(
        sysbench.opt.mysql_db, table_name, r_fmt)
    log_with_ts(stmt)
    ddl_con:query(stmt)

end

function change_column(table_name)
    local i
    local stmt
    print("Changing coloumn data type for "..table_name)
    stmt = ([[select column_name, is_nullable, data_type, CHARACTER_MAXIMUM_LENGTH, 
             NUMERIC_PRECISION, NUMERIC_SCALE, COLUMN_COMMENT 
             from information_schema.columns a 
             where a.table_name = '%s'  and a.table_schema = '%s' and a.data_type in ('varchar', 'char') and
             a.column_name not in 
             (select column_name from information_schema.KEY_COLUMN_USAGE 
               where table_schema = '%s' and table_name = '%s')]]):format(
                table_name, sysbench.opt.mysql_db, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Table has no char/varchar type")
        return
    end
            
    pos = sysbench.rand.uniform(1,rs.nrows)
    for i = 1, pos do
        row = rs:fetch_row()
    end
    col_name = tostring(row[1])
    is_null = tostring(row[2])
    data_type = tostring(row[3])
    len = tonumber(row[4])
    comment = tostring(row[7])

    if is_null == 'YES' then
        null_clause = 'NULL'
    else
        null_clause = 'NOT NULL'
    end

    if (comment == 'changed') then
        len = len - 10
        comment = ''
    else
        len = len + 10
        comment = 'changed'
    end
    stmt = ([[alter table %s.%s change column %s %s %s(%d) %s comment '%s']]):format(
        sysbench.opt.mysql_db, table_name, col_name, col_name, data_type, len, null_clause, comment)
    log_with_ts(stmt)
    ddl_con:query(stmt)
end


function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end
    
