--[[
****************************************************************************
Input parameters:

--simsim-databases = N: the number of databases to create/use (1 by default)
****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps =
       {"Desiried transaction throughput (TPS)",-1}

}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function thread_init(thread_id)
    local i, j, k, n_fks
    local stmt
    local child_keys, parent_keys
    if sysbench.opt.threads > 5 then
        error("No more than 5 threads")
    end

    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    con1 = drv:connect()
    con2 = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end

    -- Enum current schema tables
    rs = con:query("select table_name from information_schema.tables " ..
                   "where table_schema = '"..sysbench.opt.mysql_db.."'" ..
                   "and table_type = 'BASE TABLE' and table_name not like 'random_ddl%%'")
    n_table = rs.nrows
    for i = 1, n_table do
        row = rs:fetch_row()
        tables[i] = tostring(row[1])
    end

end


-- The main event loop
function event()
    con:query("USE " .. sysbench.opt.mysql_db)
    cur_table = tables[sysbench.rand.uniform(1,n_table)]

    local max_op =  17
    local op_type = sysbench.rand.uniform(1,max_op)
--    local op_type =7
    if op_type ==1 then
        op ='add_column'
    elseif op_type == 2 then
        op = "add_column_varchar"
    elseif op_type == 3 then
        op = "add_index"
    elseif op_type == 4 then
        op = "drop_index"
    elseif op_type == 5 then
        op = "add_index"
    elseif op_type == 6 then
        op = "add_prefix_index"
    elseif op_type == 7 then
        op = "add_multicol_index"
    elseif op_type == 8 then
        op = "add_index"
    elseif op_type == 9 then
        op = "optimize_table"
    elseif op_type == 10 then
        op = "drop_fulltext_index"
    elseif op_type == 11 then
        op = "add_fulltext_index"
    elseif op_type == 12 then
        op = "drop_table"
    elseif op_type == 13 then
        op  =  "create_table"
    elseif op_type == 14 then
        op  =  "create_table_as"
    elseif op_type == 15 then
        op = "create_table_like"
    elseif op_type == 16 then
        op = "create_table_longvar"
    else
        op = "rename_table"
    end

    _G[op](cur_table)
    throttle_event()
end

function optimize_table(table_name)
    print("Optimize table "..table_name)
    ddl_con:query("optimize table "..table_name)
end

function add_column(table_name)
    print("Add column to table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows >= 80 then
        print("Table already has maximum of 50 random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,99999999)
    col_name = "rand_ddl_"..col_no

    op = sysbench.rand.uniform(1,7)
    if op == 1 then
        col_def = 'int default 0 not null'
    elseif op == 2 then
        col_def = "varchar(100) default 'this is a long long test string'"
    elseif op == 3 then
        col_def = 'decimal(20,2) default 2.2'
    elseif op == 4 then
        col_def = 'double default 1'
    elseif op == 5 then
        col_def = "datetime default '2012-12-12 08:12:23'"
    elseif op == 6 then
        col_def  = "timestamp default '2012-12-12 09:12:23.000000'"
    else
        col_def = 'binary(20)'
    end

    stmt = ([[alter table %s.%s add column %s %s]]):format(
                   sysbench.opt.mysql_db, table_name, col_name, col_def)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function RandomVar(length)
	local res = ""
	for i = 1, length do
		res = res .. string.char(math.random(97, 122))
	end
	return res
end

function add_column_varchar(table_name)
    print ("Add medium varchar to table "..table_name)
    stmt =([[select column_name from information_schema.columns
              where table_schema = '%s' and table_name ='%s'
              and data_type='varchar' and character_maximum_length >= 1000
              and column_name like 'rand_ddl_%%']]):format(
        sysbench.opt.mysql_db,table_name
        )
    rs = con:query(stmt)
    if rs.nrows >=1  then
        print("Table already has long varchar column ")
        return
    end

    stmt  = ([[select column_name from information_schema.columns
             where table_schema ='%s' and table_name ='%s'
             and column_name like 'rand_ddl_%%']]):format(
            sysbench.opt.mysql_db,table_name
            )
    rs = con:query(stmt)

    col_no = rs.nrows + 1
    col_name = "rand_ddl_"..col_no

    length = sysbench.rand.uniform(1000,3000)

    defaultvalue = RandomVar(length)

    stmt = ([[alter table %s.%s add column %s varchar(%s) default '%s' ]]):format(
            sysbench.opt.mysql_db,table_name, col_name,length,defaultvalue)

    print (stmt)
--    print ("-----------------------------------------medium length varchar")
    rs = ddl_con:query(stmt)
end



function drop_column(table_name)
    print("Drop column from table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[alter table %s.%s drop column %s]]):format(
                   sysbench.opt.mysql_db, table_name, col_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end



function add_index(table_name)
    print("Add index to table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[SELECT index_name FROM information_schema.statistics
               WHERE table_schema = '%s' and table_name = '%s' and
               index_name <> 'PRIMARY' and index_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name, col_name)
    rs = con:query(stmt)

    if rs.nrows ~= 0 then
        print(([[Column %s alrady has index]]):format(col_name))
        return
    end

    stmt = ([[alter table %s.%s add index %s ( %s )]]):format(
                   sysbench.opt.mysql_db, table_name, col_name, col_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end

function add_prefix_index(table_name)
    print("Add prefix index to table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and
               column_name like 'rand_ddl_%%' and data_type='varchar'
               and character_maximum_length >=100 ]]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no random varchar columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[SELECT index_name FROM information_schema.statistics
               WHERE table_schema = '%s' and table_name = '%s' and
               index_name <> 'PRIMARY' and index_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name, col_name)
    rs = con:query(stmt)

    if rs.nrows ~= 0 then
        print(([[Column %s alrady has index]]):format(col_name))
        return
    end

    local  indexlength = sysbench.rand.uniform(10,100)
    stmt = ([[alter table %s.%s add index %s(%s(%s))]]):format(
                   sysbench.opt.mysql_db, table_name,  col_name,col_name,indexlength)
    print(stmt)
--    print ("--------------------------------------------- prefix index")
    rs = ddl_con:query(stmt)
end

function add_multicol_index(table_name)
    print("Add multi col index to table "..table_name)

    stmt = ([[SELECT index_name FROM information_schema.statistics
               WHERE table_schema = '%s' and table_name = '%s' and
               index_name like 'rand_ddl_midx%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)

    if rs.nrows >= 8 then
        print ("Table has 8 multi column index now "..table_name)
        return
    end

    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and
                data_type like 'int%%' order by ordinal_position desc limit 3]]):format(
                   sysbench.opt.mysql_db, table_name)

    rs = con:query(stmt)
    if rs.nrows < 2 then
        print("Table do not have enough quanlity columns")
        return
    end

    row = rs:fetch_row()
    col_name1 = tostring(row[1])
    row = rs:fetch_row()
    col_name2 = tostring(row[1])


    op = sysbench.rand.uniform(1,4)

    local tmp_randnum = sysbench.rand.uniform(1,1000)

    if op ==1 then
        stmt = ([[alter table %s.%s add index rand_ddl_midx_%s_%s_1_%s (%s,%s)]]):format(
                sysbench.opt.mysql_db,table_name,col_name1,col_name2,tmp_randnum,col_name1,col_name2
                )
    elseif op==2 then
        stmt = ([[alter table %s.%s add index rand_ddl_midx_%s_%s_2_%s (%s desc ,%s)]]):format(
                sysbench.opt.mysql_db,table_name,col_name1,col_name2,tmp_randnum,col_name1,col_name2
                )
    elseif op==3 then
        stmt = ([[alter table %s.%s add index rand_ddl_midx_%s_%s_3_%s (%s,%s desc)]]):format(
                sysbench.opt.mysql_db,table_name,col_name1,col_name2,tmp_randnum,col_name1,col_name2
                )
    elseif op==4 then
        stmt = ([[alter table %s.%s add index rand_ddl_midx_%s_%s_4_%s (%s desc,%s desc)]]):format(
                sysbench.opt.mysql_db,table_name,col_name1,col_name2,tmp_randnum,col_name1,col_name2
                )
    end
--    print("-----------------------------------------multicol index")
    print(stmt)
    rs = ddl_con:query(stmt)

end


function drop_index(table_name)
    print("Drop index from table "..table_name)
--    stmt = ([[select column_name from information_schema.columns
--               where table_schema = '%s' and table_name = '%s' and
--               column_name like 'rand_ddl_%%']]):format(
--                   sysbench.opt.mysql_db, table_name)
--    rs = con:query(stmt)
--    if rs.nrows == 0 then
--        print("Table has no random columns")
--        return
--    end
--
--    col_no = sysbench.rand.uniform(1,rs.nrows)
--
--    for i = 1, col_no do
--        row = rs:fetch_row()
--        col_name = tostring(row[1])
--    end

    stmt = ([[SELECT index_name FROM information_schema.statistics
               WHERE table_schema = '%s' and table_name = '%s' and
               index_name <> 'PRIMARY' and index_name like 'rand_ddl%%' ]]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)

    if rs.nrows == 0 then
        print(([[Table %s has no random index]]):format(table_name))
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        index_name = tostring(row[1])
    end

    stmt = ([[alter table %s.%s drop index %s ]]):format(
                   sysbench.opt.mysql_db, table_name, index_name)
    print(stmt)
    rs = ddl_con:query(stmt)
end



function change_row_format(table_name)
    local i, r_fmt, fl, op, stmt
    -- If table is partitioned by KEY, we can't drop the PK.
    stmt = ([[select distinct ROW_FORMAT from information_schema.tables
               where table_schema = '%s' and table_name = '%s']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    r_fmt = tostring(rs:fetch_row()[1])
    if r_fmt == 'Redundant' then
        fl = 1
    elseif r_fmt == 'Compact' then
        fl = 2
    elseif r_fmt == 'Compressed' or r_fmt == 'Dynamic' then
        fl = 3
    end

    op = sysbench.rand.uniform(fl,4)
    if op == 1 then
        r_fmt = 'Redundant'
    elseif op == 2 then
        r_fmt = 'Compact'
    elseif op == 3 then
        r_fmt = 'Compressed'
    else
        r_fmt = 'Dynamic'
    end

    stmt = ([[alter table %s.%s row_format = %s]]):format(
        sysbench.opt.mysql_db, table_name, r_fmt)
    print(stmt)
    ddl_con:query(stmt)

end

function change_column(table_name)
    local i
    local stmt
    print("Changing coloumn data type for "..table_name)
    stmt = ([[select column_name, is_nullable, data_type, CHARACTER_MAXIMUM_LENGTH,
             NUMERIC_PRECISION, NUMERIC_SCALE, COLUMN_COMMENT
             from information_schema.columns a
             where a.table_name = '%s'  and a.table_schema = '%s' and a.data_type in ('varchar', 'char') and
             a.column_name not in
             (select column_name from information_schema.KEY_COLUMN_USAGE
               where table_schema = '%s' and table_name = '%s')]]):format(
                table_name, sysbench.opt.mysql_db, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no char/varchar type")
        return
    end

    pos = sysbench.rand.uniform(1,rs.nrows)
    for i = 1, pos do
        row = rs:fetch_row()
    end
    col_name = tostring(row[1])
    is_null = tostring(row[2])
    data_type = tostring(row[3])
    len = tonumber(row[4])
    comment = tostring(row[7])

    if is_null == 'YES' then
        null_clause = 'NULL'
    else
        null_clause = 'NOT NULL'
    end

    if (comment == 'changed') then
        len = len - 10
        comment = ''
    else
        len = len + 10
        comment = 'changed'
    end
    stmt = ([[alter table %s.%s change column %s %s %s(%d) %s comment '%s']]):format(
        sysbench.opt.mysql_db, table_name, col_name, col_name, data_type, len, null_clause, comment)
    print(stmt)
    ddl_con:query(stmt)
end

function alter_column_with_index(table_name)
    local i
    local stmt
    print("Changing coloumn data type for "..table_name)
    stmt = ([[select column_name, is_nullable, data_type, CHARACTER_MAXIMUM_LENGTH,
             NUMERIC_PRECISION, NUMERIC_SCALE, COLUMN_COMMENT
             from information_schema.columns a
             where a.table_name = '%s'  and a.table_schema = '%s' and a.data_type in ('varchar', 'char') and
             a.column_name not in
             (select column_name from information_schema.KEY_COLUMN_USAGE
               where table_schema = '%s' and table_name = '%s')]]):format(
                table_name, sysbench.opt.mysql_db, sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no char/varchar type")
        return
    end

    pos = sysbench.rand.uniform(1,rs.nrows)
    for i = 1, pos do
        row = rs:fetch_row()
    end
    col_name = tostring(row[1])
    is_null = tostring(row[2])
    data_type = tostring(row[3])
    len = tonumber(row[4])
    comment = tostring(row[7])

    if is_null == 'YES' then
        null_clause = 'NULL'
    else
        null_clause = 'NOT NULL'
    end

    if (comment == 'changed') then
        len = len - 10
        comment = ''
    else
        len = len + 10
        comment = 'changed'
    end
    stmt = ([[alter table %s.%s change column %s %s %s(%d) %s comment '%s']]):format(
        sysbench.opt.mysql_db, table_name, col_name, col_name, data_type, len, null_clause, comment)
    print(stmt)
    ddl_con:query(stmt)
end

function drop_fulltext_index(table_name)
    stmt = ([[select index_name from information_schema.statistics
               where table_schema = '%s' and table_name = '%s' and
               index_type='FULLTEXT']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no fulltext index")
        return
    end

    row = rs:fetch_row()
    index_name = tostring(row[1])
    stmt = ([[alter table %s.%s drop index %s ]]):format(
                   sysbench.opt.mysql_db, table_name, index_name)
    print(stmt)
    rs = ddl_con:query(stmt)

end


function add_fulltext_index(table_name)

    print("add fulltext index for "..table_name)

    stmt = ([[ select * from information_schema.partitions
            where table_schema = '%s' and table_name = '%s'
            and PARTITION_METHOD is not NULL
            ]]):format(sysbench.opt.mysql_db,table_name)
    rs = con:query(stmt)
    if rs.nrows ~= 0  then
		print(([[Table %s is partition table]]):format(table_name))
		return
    end

    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and
               column_name like 'rand_ddl_%%' and column_type like 'varchar%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then
        print("Table has no random varchar columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
        index_name = "fulltext_" .. col_name
    end

    stmt = ([[SELECT index_name FROM information_schema.statistics
               WHERE table_schema = '%s' and table_name = '%s' and
               index_name <> 'PRIMARY' and index_name = '%s' and
               index_type='FULLTEXT']]):format(
                   sysbench.opt.mysql_db, table_name, index_name)
    rs = con:query(stmt)

    if rs.nrows ~= 0 then
        print(([[Column %s alrady has fulltext index]]):format(col_name))
        return
    end

    stmt = ([[alter table %s.%s add fulltext index %s ( %s )]]):format(
                   sysbench.opt.mysql_db, table_name, index_name, col_name)
    print(stmt)
    rs = ddl_con:query(stmt)

end



function create_table_like(table_name)
    print("create table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name ='random_ddl_%s']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl_"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s like %s.%s ]]):format(
            sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
            )
    print(stmt)
    rs = ddl_con:query(stmt)
end


function create_table_as(table_name)
    print("create table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name ='random_ddl_%s']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl_"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s as select * from %s.%s ]]):format(
            sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
            )
    print(stmt)
    rs = ddl_con:query(stmt)
end

function create_table(table_name)
    print("create table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name ='random_ddl_%s']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s(col1 int,col2 int,col3 int,col4 int,col5 int,
                col6 int,col7 int,col8 int,col9 varchar(10),col10 varchar(100))]]
            ):format(sysbench.opt.mysql_db,table_name)
    print(stmt)
    rs = ddl_con:query(stmt)

end

function create_table_longvar(table_name)
    print("create table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name ='random_ddl_%s']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s (col1 int,col2 int,col3 int,col4 int,col5 int,
                col6 int,col7 int,col8 int,col9 varchar(1000),col10 varchar(1000)
                ) ]]):format(
            sysbench.opt.mysql_db,table_name
            )
    print(stmt)
    rs = ddl_con:query(stmt)
end

function rename_table(table_name)
    print("rename table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name ='random_ddl_%s']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  == 0 then
        print ("Do not have table random_ddl_"..table_name)
        return
    end

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name ='random_ddl_%s_rn']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl_"..table_name.."_rn")
        return
    end

    stmt = ([[alter table %s.random_ddl_%s rename to %s.random_ddl_%s_rn ]]):format(
            sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
            )
    print(stmt)
    rs = ddl_con:query(stmt)
end

function drop_table(table_name)
    print("drop table ")

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name like 'random_ddl_%s%%']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  == 0 then
        print ("Do not have table to drop random_ddl_"..table_name)
        return
    end

    row = rs:fetch_row()
    droptbname = tostring(row[1])

    stmt = ([[drop table %s.%s]]):format(
            sysbench.opt.mysql_db,droptbname
            )
    print(stmt)
    rs = ddl_con:query(stmt)

end

function force_rebuild(table_name)
    print("force table "..table_name .." rebuild")
    stmt=([[alter table %s.%s engine=innodb]]):format(sysbench.opt.mysql_db, table_name)
    ddl_con:query(stmt)
end


function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end


