--[[
****************************************************************************
Input parameters:

--simsim-databases = N: the number of databases to create/use (1 by default)
****************************************************************************
--]]

require("timed_event")

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: run")
end

-- Command line options
sysbench.cmdline.options = {
    intrusive =
       {"Include intrusive DDL operations", "no"},
    report_csv =
       {"Report output in csv (yes, no)", "no"},
    target_tps =
       {"Desiried transaction throughput (TPS)",-1}

}
tables = {}
n_table = 0

function sleep(n)
    os.execute("sleep " .. tonumber(n))
end

function log_with_ts(str)
    print(([[%s(%s) %s]]):format(os.date("%Y-%m-%dT%T"), os.time(), str))
end

function thread_init(thread_id)
    local i, j, k, n_fks
    local stmt
    local child_keys, parent_keys
    if sysbench.opt.threads > 4 then
        error("No more than 4 threads")
    end

    timed_event_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
    con1 = drv:connect()
    con2 = drv:connect()
    ddl_con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end

    -- Enum current schema tables
    rs = con:query("select table_name from information_schema.tables " ..
                   "where table_schema = '"..sysbench.opt.mysql_db.."'" ..
                   "and table_type = 'BASE TABLE' and table_name not like 'random_ddl%' and table_name not like '%_new' and table_name not like '%_old'")
    n_table = rs.nrows
    for i = 1, n_table do
        row = rs:fetch_row()
        tables[i] = tostring(row[1])
    end

end


-- The main event loop
function event()
    con:query("USE " .. sysbench.opt.mysql_db)
    cur_table = tables[sysbench.rand.uniform(1,n_table)]

    local max_op =  8
    local op_type = sysbench.rand.uniform(1,max_op)
    
    if op_type == 1 then
        op = "create_table_like"
    elseif op_type == 2 then
        op = "create_table_as"
    elseif op_type == 3 then
        op = "create_table"
    elseif op_type == 4 then
        op = "create_table_longvar"
    elseif op_type == 5 then
        op = "rename_table"
    elseif op_type == 6 then
        op = "replace_table"
    elseif op_type == 7 then
        op = "rename_table_2"
        print("bingo")
    else
        op = "drop_table"
    end

    _G[op](cur_table)
    throttle_event()
end

function create_table_like(table_name)
    print("create table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name in ('random_ddl_%s','random_ddl_%s_rn')]]):format(
            sysbench.opt.mysql_db,table_name,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl_"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s like %s.%s ]]):format(
            sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
            )
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)

    stmt = ([[insert into %s.random_ddl_%s select * from %s.%s limit 100 ]]):format(
            sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
            )
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)

end


function create_table_as(table_name)
    print("create table random_ddl_"..table_name.." as ..")

    stmt="select @@log_bin log_bin, @@enforce_gtid_consistency enforce_gtid_consistency"
    rs = con:query(stmt)
    row = rs:fetch_row()
    log_bin = tonumber(row[1])
    enforce_gtid_consistency = tostring(row[2])
    
    if (log_bin==1 and enforce_gtid_consistency == 'ON') then
        print ("Skpping create table .. as when binlog is on")
        return
    end

    stmt=([[select table_name from information_schema.tables
            where table_schema = '%s' and table_name in ('random_ddl_%s','random_ddl_%s_rn')]]):format(
                sysbench.opt.mysql_db,table_name,table_name
     )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl_"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s as select * from %s.%s limit 100]]):format(
            sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
            )
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)

end

function create_table(table_name)
    print("create table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
            where table_schema = '%s' and table_name in ('random_ddl_%s','random_ddl_%s_rn')]]):format(
                sysbench.opt.mysql_db,table_name,table_name
     )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s(col1 int,col2 int,col3 int,col4 int,col5 int,
                col6 int,col7 int,col8 int,col9 varchar(10),col10 varchar(100))]]
            ):format(sysbench.opt.mysql_db,table_name)
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)

end

function create_table_longvar(table_name)
    print("create table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
            where table_schema = '%s' and table_name in ('random_ddl_%s','random_ddl_%s_rn')]]):format(
                sysbench.opt.mysql_db,table_name,table_name
     )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl"..table_name)
        return
    end

    stmt = ([[create table  %s.random_ddl_%s (col1 int,col2 int,col3 int,col4 int,col5 int,
                col6 int,col7 int,col8 int,col9 varchar(1000),col10 varchar(1000)
                ) ]]):format(
            sysbench.opt.mysql_db,table_name
            )
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)
end

function rename_table(table_name)
    print("rename table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
            where table_schema = '%s' and table_name in ('random_ddl_%s','random_ddl_%s_rn')]]):format(
                sysbench.opt.mysql_db,table_name,table_name
     )
    rs = con:query(stmt)

    if rs.nrows  == 0 then
        print ("Do not have table random_ddl_"..table_name)
        return
    end

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name ='random_ddl_%s_rn']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  > 0 then
        print ("Already have random_ddl_"..table_name.."_rn, rename it back")
        stmt = ([[alter table %s.random_ddl_%s_rn rename to %s.random_ddl_%s ]]):format(
                sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
                )
        log_with_ts(stmt)
        rs = ddl_con:query(stmt)
    else
        stmt = ([[alter table %s.random_ddl_%s rename to %s.random_ddl_%s_rn ]]):format(
                sysbench.opt.mysql_db,table_name,sysbench.opt.mysql_db,table_name
                )
        log_with_ts(stmt)
        rs = ddl_con:query(stmt)
    end
end

function replace_table(table_name)
    print("replace table random_ddl_"..table_name)

    stmt=([[select table_name from information_schema.tables
            where table_schema = '%s' and table_name in ('random_ddl_%s')]]):format(
                sysbench.opt.mysql_db,table_name,table_name
     )
    rs = con:query(stmt)

    if rs.nrows  == 0 then
        print ("Do not have table random_ddl_"..table_name)
        return
    end

    rename_table("random_ddl_"..table_name)
end

function drop_table(table_name)
    print("drop table ")

    stmt=([[select table_name from information_schema.tables
           where table_schema = '%s' and table_name like 'random_ddl_%s%%']]):format(
            sysbench.opt.mysql_db,table_name
         )
    rs = con:query(stmt)

    if rs.nrows  == 0 then
        print ("Do not have table to drop random_ddl_"..table_name)
        return
    end

    row = rs:fetch_row()
    droptbname = tostring(row[1])

    stmt = ([[drop table %s.%s]]):format(
            sysbench.opt.mysql_db,droptbname
            )
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)

end

function rename_table_2(table_name)
    local i, r_fmt, fl, op, stmt

    if  sysbench.opt.intrusive == "no" then
        return
    end

    stmt = string.format('drop table if exists %s.%s_new ', 
                         sysbench.opt.mysql_db, table_name)
    log_with_ts(stmt)
    ddl_con:query(stmt)

    stmt = string.format('drop table if exists %s.%s_old ', 
    sysbench.opt.mysql_db, table_name)
    log_with_ts(stmt)
    ddl_con:query(stmt)

    stmt = string.format('create table %s.%s_new like %s.%s', 
                         sysbench.opt.mysql_db, table_name,
                         sysbench.opt.mysql_db, table_name)
    print(stmt)
    ddl_con:query(stmt)

    stmt = ([[insert into %s.%s_new select * from %s.%s limit 100]]):format(
        sysbench.opt.mysql_db, table_name,sysbench.opt.mysql_db, table_name)
    log_with_ts(stmt)
    ddl_con:query(stmt)

    op = sysbench.rand.uniform(1,2)
    if op == 1 then
        add_column(("%s_new"):format(table_name))
    elseif op == 2 then
        drop_column(("%s_new"):format(table_name))
    end

    stmt = ([[rename table %s.%s to %s.%s_old, %s.%s_new to %s.%s]]):format(
        sysbench.opt.mysql_db, table_name, sysbench.opt.mysql_db, table_name, 
        sysbench.opt.mysql_db, table_name, sysbench.opt.mysql_db, table_name)
    log_with_ts(stmt)
    ddl_con:query(stmt)

    stmt = ([[drop table %s.%s_old]]):format(
        sysbench.opt.mysql_db, table_name)
    log_with_ts(stmt)
    ddl_con:query(stmt)


end

function add_column(table_name)
    print("Add column to table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and 
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows >=50 then 
        print("Table already has maximum of 50 random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,99999999)
    col_name = "rand_ddl_"..col_no

    op = sysbench.rand.uniform(1,6)
    if op == 1 then
        col_def = 'int default 0 not null'
    elseif op == 2 then
        col_def = "varchar(100) default 'this is a long long test string'"
    elseif op == 3 then
        col_def = 'decimal(20,2) default 2.2'
    elseif op == 4 then
        col_def = 'double default 1'
    elseif op == 5 then
        col_def = "datetime default '2012-12-12 08:12:23'"
    else
        col_def = 'binary(20)'
    end

    stmt = ([[alter table %s.%s add column %s %s]]):format(
                   sysbench.opt.mysql_db, table_name, col_name, col_def)
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)
end

function drop_column(table_name)
    print("Drop column from table "..table_name)
    stmt = ([[select column_name from information_schema.columns
               where table_schema = '%s' and table_name = '%s' and 
               column_name like 'rand_ddl_%%']]):format(
                   sysbench.opt.mysql_db, table_name)
    rs = con:query(stmt)
    if rs.nrows == 0 then 
        print("Table has no random columns")
        return
    end

    col_no = sysbench.rand.uniform(1,rs.nrows)

    for i = 1, col_no do
        row = rs:fetch_row()
        col_name = tostring(row[1])
    end

    stmt = ([[alter table %s.%s drop column %s]]):format(
                   sysbench.opt.mysql_db, table_name, col_name)
    log_with_ts(stmt)
    rs = ddl_con:query(stmt)
end

function sysbench.hooks.before_restart_event(err)
  sleep(1)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    -- --   print("my stat: ", val)
       if  sysbench.opt.report_csv == "yes" then
           sysbench.report_csv(stat)
       else
           sysbench.report_default(stat)
       end
end


