--
-- Created by IntelliJ IDEA.
-- User: zhanglu
-- Date: 2018/12/7
-- Time: 13:44
-- To change this template use File | Settings | File Templates.


require("oltp_common")

-- Command line options
sysbench.cmdline.options.field_size = {"Bytes of the table field pad", 20 }
sysbench.cmdline.options.rows = {"Rows num of results", 100 }
sysbench.cmdline.options.table_name = {"Name of table", "sbtest" }

sysbench.cmdline.commands.prepare = {
    function ()
        do_prepare()
    end,
    sysbench.cmdline.PARALLEL_COMMAND
}

function do_prepare()
    assert(sysbench.opt.table_size >= sysbench.opt.rows, "this benchmark does not support -- table_rows < result_set_rows")

    local drv = sysbench.sql.driver()
    local con = drv:connect()
    field_size = sysbench.opt.field_size
    table_name = sysbench.opt.table_name

    for i = sysbench.tid % sysbench.opt.threads + 1, sysbench.opt.tables,
    sysbench.opt.threads do
        create_table_me(drv, con, i, field_size, table_name)
    end
end

function cleanup()

    local drv = sysbench.sql.driver()
    local con = drv:connect()
    table_name = sysbench.opt.table_name
  
    print("tables count: ", sysbench.opt.tables)

    for i = 1, sysbench.opt.tables do
        print(string.format("Dropping table '%s%d'...", table_name, i))
        con:query("DROP TABLE IF EXISTS " .. table_name .. i )
    end

end

function create_table_me(drv, con, table_num, field_size, table_name)
    local id_index_def, id_def
    local query

    if sysbench.opt.secondary then
        id_index_def = "KEY xid"
    else
        id_index_def = "PRIMARY KEY"
    end

    if drv:name() == "mysql"
    then
        if sysbench.opt.auto_inc then
            id_def = "INTEGER NOT NULL AUTO_INCREMENT"
        else
            id_def = "INTEGER NOT NULL"
        end
        engine_def = "/*! ENGINE = " .. sysbench.opt.mysql_storage_engine .. " */"
    elseif drv:name() == "pgsql"
    then
        if not sysbench.opt.auto_inc then
            id_def = "INTEGER NOT NULL"
        elseif pgsql_variant == 'redshift' then
            id_def = "INTEGER IDENTITY(1,1)"
        else
            id_def = "SERIAL"
        end
        engine_def = ""
    else
        error("Unsupported database driver:" .. drv:name())
    end

    print(string.format("Creating table '%s%d'...", table_name, table_num))

    if drv:name() == "pgsql"
    then
        query = string.format([[
    CREATE TABLE %s%d(
      id %s,
      k INTEGER DEFAULT '0' NOT NULL,
      c CHAR(120) DEFAULT '' NOT NULL,
      pad TEXT DEFAULT '' NOT NULL,
      %s (id)
    ) %s %s]],
            table_name, table_num, id_def, id_index_def, engine_def,
            sysbench.opt.create_table_options)
    else
        query = string.format([[
    CREATE TABLE %s%d(
      id %s,
      k INTEGER DEFAULT '0' NOT NULL,
      c CHAR(120) DEFAULT '' NOT NULL,
      pad TEXT(%d) DEFAULT '' NOT NULL,
      %s (id)
    ) %s %s]],
            table_name, table_num, id_def, field_size, id_index_def, engine_def,
            sysbench.opt.create_table_options)
    end

    con:query(query)

    if (sysbench.opt.table_size > 0) then
        print(string.format("Inserting %d records into '%s%d'",
            sysbench.opt.table_size, table_name, table_num))
    end

    if sysbench.opt.auto_inc then
        query = "INSERT INTO " .. table_name .. table_num .. "(k, c, pad) VALUES"
    else
        query = "INSERT INTO " .. table_name .. table_num .. "(id, k, c, pad) VALUES"
    end

    con:bulk_insert_init(query)

    local c_val
    local pad_val


    for i = 1, sysbench.opt.table_size do

        c_val = get_c_value()
        pad_val = sysbench.rand.string(string.rep("@", field_size))

        if (sysbench.opt.auto_inc) then
            query = string.format("(%d, '%s', '%s')",
                sysbench.rand.uniform(1, sysbench.opt.table_size),
                c_val, pad_val)
        else
            query = string.format("(%d, %d, '%s', '%s')",
                i,
                sysbench.rand.uniform(1, sysbench.opt.table_size),
                c_val, pad_val)
        end

        con:bulk_insert_next(query)
    end

    con:bulk_insert_done()

    if sysbench.opt.create_secondary then
        print(string.format("Creating a secondary index on '%s%d'...",
            table_name, table_num))
        con:query(string.format("CREATE INDEX k_%d ON %s%d(k)",
            table_num, table_name, table_num))
    end
end


function thread_init()

    drv = sysbench.sql.driver()
    con = drv:connect()
end


function event(thread_id)
    beginning = sysbench.rand.uniform(1, (sysbench.opt.table_size - sysbench.opt.rows + 1))
    ending = beginning + sysbench.opt.rows - 1
    table_name = sysbench.opt.table_name
    sql = string.format("SELECT pad FROM %s%d where id BETWEEN %s AND %s",
        table_name, math.min((thread_id % sysbench.opt.threads + 1), sysbench.opt.tables),
        beginning, ending)
    con:query(sql)
end

function thread_done()
   if con then
       con:disconnect()
   end
end

function sysbench.hooks.before_restart_event(err)
    con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
    if  sysbench.opt.report_csv == "yes" then
        sysbench.report_csv(stat)
    else
        sysbench.report_default(stat)
    end
end



