-- sysbench --mysql-host=127.0.0.1 --mysql-port=2250 --mysql-user=root
--          --mysql-password=Passw0rd --db-driver=mysql short_con.lua
--          --threads=1 --time=5 --mysql-db=test --report-interval=1
--          --target_tps=5 --ignore-mysql-errors=2003 run

require("timed_event")
ffi = require("ffi")

ffi.cdef[[
  void sb_counter_inc(int, sb_counter_type);
  void sb_counter_dec(int, sb_counter_type);
]]

before_pnano = assert(ffi.new("nanotime[?]", 1))
after_pnano = assert(ffi.new("nanotime[?]", 1))


if sysbench.cmdline.command == nil then
   error("Command is required. Supported commands: prepare, run, cleanup, help")
end

-- Command line options
sysbench.cmdline.options = {
  skip_trx =
      {"Don't start explicit transactions and execute all queries " ..
       "in the AUTOCOMMIT mode, default is False.", false},
  max_reconnect =
       {"How many times to try to connect before failing " ..
        "the test, default is 10.  Reconnect interval is 1 second.", 10},
  target_tps =
      {"Desired transaction throughput (trx/sec). Default(-1) is no limit.", -1}
}

function thread_init()
  drv = sysbench.sql.driver()
  timed_event_init(thread_id)
end

function prepare()
  print("Running prepare")
end

function elapsed_time(before, after)  --Result is in milliseconds(1/1000 s)
  return tonumber(after[0].tv_sec*1000) + math_floor(tonumber(after[0].tv_nsec/1000000)) -
         tonumber(before[0].tv_sec*1000) + math_floor(tonumber(before[0].tv_nsec/1000000))
end


function event()
  local conn = nil
  local retry = 0

  ffi.C.clock_gettime(1, before_pnano)
  conn=ffi.C.db_connection_create(drv)
  ffi.C.clock_gettime(1, after_pnano)
  if conn == nil then
    print("Last failed connect attempt took "..elapsed_time(before_pnano, after_pnano).. " ms, trying to reconnect")
  end

  while (conn == nil and retry < sysbench.opt.max_reconnect) do  --retry for 5 minutes
    ffi.C.usleep(1000000) -- sleep 1 second before retry
    ffi.C.clock_gettime(1, before_pnano)
    conn=ffi.C.db_connection_create(drv)
    ffi.C.clock_gettime(1, after_pnano)
    if conn == nil then
      print("Last failed connect attempt took "..elapsed_time(before_pnano, after_pnano).. " ms, trying to reconnect")
    end
    retry = retry + 1
  end

  -- ffi.C.usleep(20000) --Inject some dely to test
  if conn == nil then
    print("Max connect retry time ("..sysbench.opt.max_reconnect..") reached, aborting")
    ffi.C.sb_counter_inc(sysbench.tid, ffi.C.SB_CNT_ERROR)
    ffi.C.sb_counter_dec(sysbench.tid, ffi.C.SB_CNT_TRX)
  -- os.exit()
  -- else
  --   total_connect_count = total_connect_count + 1
  --   total_connect_time = elapsed_time(before_pnano, after_pnano)
  
  else
    if not sysbench.opt.skip_trx and drv:name()=="mysql" then
      conn:query("SET autocommit=0")
    end
    conn:query("select 1")
    conn:disconnect()
    ffi.C.db_connection_free(conn)
    throttle_event(thread_id) --Calculate the time delay to achieve target TPS
  end
end

