--
-- Created by IntelliJ IDEA.
-- User: zhanglu
-- Date: 2018/12/7
-- Time: 17:13
-- To change this template use File | Settings | File Templates.

require("timed_event")
require("select_multirows")

ffi = require("ffi")

ffi.cdef[[
  void sb_counter_inc(int, sb_counter_type);
  void sb_counter_dec(int, sb_counter_type);
]]


-- Command line options
sysbench.cmdline.options.skip_trx =
    {"Don't start explicit transactions and execute all queries in the AUTOCOMMIT mode", false }

sysbench.cmdline.options.target_tps =
    {"Desired transaction throughput (trx/sec)", -1 }

sysbench.cmdline.options.max_reconnect =
    {"How many times to try to connect before failing the test, default is 10.  Reconnect interval is 1 second.", 10 }

sysbench.cmdline.options.table_name =
    {"Name of table", "sbtest" }

function thread_init(thread_id)
    drv = sysbench.sql.driver()
    timed_event_init(thread_id)
end

function event(thread_id)
    conn = nil
    retry = 0
    while (conn == nil and retry < sysbench.opt.max_reconnect) do
        conn=ffi.C.db_connection_create(drv)
        if conn == nil then       -- connection failure
            ffi.C.usleep(1000000) -- sleep 1 second before retry
            retry = retry + 1
        end
    end
    if conn == nil then
        print("Max connect retry time ("..sysbench.opt.max_reconnect..") reached, aborting")
        ffi.C.sb_counter_inc(sysbench.tid, ffi.C.SB_CNT_ERROR)
        ffi.C.sb_counter_dec(sysbench.tid, ffi.C.SB_CNT_TRX)
    else
        if not sysbench.opt.skip_trx then
            conn:query("SET autocommit=0")
        end
        beginning = sysbench.rand.uniform(1, (sysbench.opt.table_size - sysbench.opt.rows + 1))
        ending = beginning + sysbench.opt.rows - 1
        table_name = sysbench.opt.table_name
        sql = string.format("SELECT pad FROM %s%d where id BETWEEN %s AND %s",
            table_name, math.min((thread_id % sysbench.opt.threads + 1), sysbench.opt.tables),
            beginning, ending)
        conn:query(sql)
        conn:disconnect()
        ffi.C.db_connection_free(conn)

        throttle_event(thread_id) --Calculate the time delay to achieve target TPS
    end
end


