--[[
****************************************************************************
Input parameters:

--simsim-databases = N: the number of databases to create/use (1 by default)
****************************************************************************
--]]

if sysbench.cmdline.command == nil then
    error("Command is required. Supported commands: prepare, run, cleanup, help")
end

-- Command line options
sysbench.cmdline.options = {
    simsim_databases =
       {"Number of databases", 1},
    report_csv =
       {"Report output in csv (yes, no)", "no"}
 
}


function prepare()

   local drv = sysbench.sql.driver()
   local con = drv:connect()

   for i = 1, sysbench.opt.simsim_databases do
      local database_name = get_database_name(i)
      con:query("DROP DATABASE IF EXISTS " .. database_name)
      con:query("CREATE DATABASE " .. database_name)
      create_tables(con,i)
   end
end

function cleanup()

   drv = sysbench.sql.driver()
   con = drv:connect()
   for i = 1, sysbench.opt.simsim_databases do
    con:query("DROP DATABASE " .. get_database_name(i))
   end
end

function thread_init(thread_id)
    drv = sysbench.sql.driver()
    con = drv:connect()

    if drv:name() == "mysql" then
      con:query("SET autocommit=0")
    end
end


-- The main event loop
function event(thread_id)

   con:query("USE " .. get_database_name(sysbench.rand.uniform(1, sysbench.opt.simsim_databases)))

   con:query("BEGIN")

   trade_no = new_trade_no()
   out_trade_no = StrRand(19)

   insert_into_trade_base(trade_no, out_trade_no)

   for i = 1,7 do
      con:query(string.format("SELECT * FROM trade_base WHERE trade_no='%s'",
                             trade_no))
   end

   insert_into_trade_note(trade_no)
   insert_into_timeout(trade_no)
   insert_into_trade_logistics(trade_no)
   insert_into_trade_process(trade_no)
   insert_into_trade_goods_info(trade_no)
   insert_into_trade_outtrade(trade_no, out_trade_no)
   insert_into_exterface_invoke()
   insert_into_fund_flow_info(trade_no)
   insert_into_trade_fund_bill(trade_no)
   insert_into_trade_ext(trade_no)
   insert_into_notify()

   for i = 1,3 do
      con:query(string.format("SELECT * FROM trade_base WHERE trade_no='%s'",
                             trade_no))
   end

   for i = 1,3 do
      con:query(string.format("SELECT * FROM trade_logistics WHERE trade_no ='%s'",
                             trade_no))
   end

   for i = 1,3 do
      con:query(string.format("SELECT * FROM trade_process WHERE trade_no ='%s'",
                             trade_no))
   end

   for i = 1,3 do
      con:query(string.format("SELECT * FROM timeout WHERE trade_no ='%s'",
                             trade_no))
   end

   for i = 1,3 do
      con:query(string.format("SELECT NOW()"))
   end

   for i = 1,3 do
      con:query(string.format("SELECT * FROM trade_ext WHERE trade_no ='%s'",
                             trade_no))
   end

   con:query("COMMIT")
end

function get_database_name(database_id)
   return string.format("trade_%02d", database_id)
end

-- Create tables in a given database.
function create_tables(con,database_id)
   con:query("USE " .. get_database_name(database_id))

   con:query([[
CREATE TABLE `exterface_invoke` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT,
  `PARTNER_ID` varchar(32) NOT NULL,
  `TARGET` varchar(64) NOT NULL,
  `EXTERFACE` varchar(128) NOT NULL,
  `NOTIFY_URL` varchar(200) DEFAULT NULL,
  `RETURN_URL` varchar(200) DEFAULT NULL,
  `SIGN_TYPE` varchar(10) DEFAULT NULL,
  `CHARSET` varchar(20) DEFAULT NULL,
  `GMT_INVOKE` datetime NOT NULL,
  `GMT_FINISH` datetime NOT NULL,
  `TARGET_TYPE` varchar(20) NOT NULL,
  `GMT_NOTIFY` datetime DEFAULT NULL,
  `GMT_RETURN` datetime DEFAULT NULL,
  PRIMARY KEY (`id`),
  key ind_exterface_invoke_tgt_tt(target, target_type, partner_id)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `fund_flow_info` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT primary key,
  `ORDER_NO` varchar(64) NOT NULL,
  `TRANS_ACCOUNT_OUT` varchar(20) NOT NULL,
  `TRANS_ACCOUNT_IN` varchar(20) DEFAULT NULL,
  `ACC_TYPE` varchar(32) NOT NULL,
  `FUND_CLEAR_FEE` bigint(15) NOT NULL,
  `PAY_TOOL_TYPE` varchar(32) NOT NULL,
  `PAY_TOOL_CONTEXT` varchar(1024) NOT NULL,
  `GMT_CREATE` datetime NOT NULL,
  `GMT_MODIFIED` datetime NOT NULL,
  unique KEY (`ORDER_NO`)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `notify` (
  `ID` bigint NOT NULL auto_increment,
  `PLUGIN` varchar(50) NOT NULL,
  `PARAMETER` varchar(4000) NOT NULL,
  `GROUP_NO` bigint(2) NOT NULL,
  `STATUS` char(1) NOT NULL,
  `GMT_CREATE` datetime NOT NULL,
  `GMT_MODIFY` datetime NOT NULL,
  `IW_USER_ID` varchar(16) DEFAULT NULL,
  `CHANNEL_ENDPOINT` varchar(200) DEFAULT NULL,
  `BUSINESS_ORDER` varchar(128) DEFAULT NULL,
  `BUSINESS_TYPE` varchar(50) DEFAULT NULL,
  `PLUGIN_HOST` varchar(15) DEFAULT NULL,
  `GMT_LOCAL_CREATE` datetime DEFAULT NULL,
  PRIMARY KEY (`ID`)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `timeout` (
  `id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
  `JOB_ID` varchar(32) NOT NULL,
  `TRADE_NO` varchar(64) DEFAULT NULL,
  `USER_ID` varchar(128) DEFAULT NULL,
  `PAYTYPE` varchar(4) DEFAULT NULL,
  `END_TIME` datetime DEFAULT NULL,
  `ACTION` varchar(30) DEFAULT NULL,
  `PARAMETER` varchar(100) DEFAULT NULL,
  `STATUS` char(1) DEFAULT NULL,
  `ACTION_TYPE` char(1) DEFAULT NULL,
  `GMT_CREATE` datetime DEFAULT NULL,
  `PRIOR_LEVEL` bigint(11) DEFAULT NULL,
  `GMT_MODIFIED` datetime DEFAULT NULL,
  PRIMARY KEY (`id`),
  KEY `JOB_ID` (`JOB_ID`),
  key ind_beyond_timeout_etime (end_time),
  key timeout_pl_ind (prior_level),
  key timeout_tno_ind (trade_no)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])


   con:query([[
CREATE TABLE `trade_base` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT,
  `OUT_TRADE_NO` varchar(64) DEFAULT NULL,
  `TRADE_NO` varchar(64) NOT NULL,
  `SELLER_ACCOUNT` varchar(100) NOT NULL,
  `SELLER_LOGIN_EMAIL` varchar(100) NOT NULL,
  `BUYER_ACCOUNT` varchar(100) NOT NULL,
  `BUYER_LOGIN_EMAIL` varchar(100) NOT NULL,
  `SELLER_TYPE` bigint(2) DEFAULT NULL,
  `BUYER_TYPE` bigint(2) DEFAULT NULL,
  `TRADE_FROM` varchar(100) DEFAULT NULL,
  `TRADE_EMAIL` varchar(100) DEFAULT NULL,
  `OPERATOR_ROLE` char(1) NOT NULL,
  `TRADE_STATUS` varchar(50) NOT NULL,
  `TOTAL_FEE` bigint(15) NOT NULL,
  `SERVICE_FEE_RATIO` int(11) DEFAULT NULL,
  `SERVICE_FEE` bigint(15) NOT NULL,
  `CURRENCY` varchar(3) DEFAULT NULL,
  `SELLER_ACTION` varchar(200) DEFAULT NULL,
  `BUYER_ACTION` varchar(200) DEFAULT NULL,
  `GMT_CREATE` datetime NOT NULL,
  `SELLER_USER_ID` varchar(16) DEFAULT NULL,
  `BUYER_USER_ID` varchar(16) DEFAULT NULL,
  `ADDITIONAL_TRD_STATUS` varchar(50) DEFAULT NULL,
  `TRADE_TYPE` varchar(4) NOT NULL,
  `SELLER_FULLNAME` varchar(256) DEFAULT NULL,
  `BUYER_FULLNAME` varchar(256) DEFAULT NULL,
  `SELLER_NICK` varchar(100) NOT NULL,
  `BUYER_NICK` varchar(100) NOT NULL,
  `GOODS_TITLE` varchar(256) DEFAULT NULL,
  `GMT_LAST_MODIFIED_DT` datetime DEFAULT NULL,
  `STOP_TIMEOUT` varchar(4) DEFAULT NULL,
  `GATHERING_TYPE` varchar(4) DEFAULT NULL,
  `BUYER_MARKER` char(1) DEFAULT NULL,
  `SELLER_MARKER` char(1) DEFAULT NULL,
  `BUYER_MARKER_MEMO` varchar(1000) DEFAULT NULL,
  `SELLER_MARKER_MEMO` varchar(1000) DEFAULT NULL,
  `CHANNEL` varchar(32) DEFAULT NULL,
  `PRODUCT` varchar(20) DEFAULT NULL,
  `PAY_CHANNEL` varchar(32) DEFAULT NULL,
  `OTHER_SERVICE_FEE` bigint(15) DEFAULT NULL,
  `RELATION_PRO` varchar(64) DEFAULT NULL,
  PRIMARY KEY (`id`),
  key byd_trade_bse_ind(trade_email, gmt_create),
  key ind_trade_base_00_otrdno  (out_trade_no),
  unique key pk_trade_base_00 (trade_no)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `trade_ext` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT primary key,
  `TRADE_NO` varchar(64) NOT NULL,
  `EXT_INFO` varchar(400) DEFAULT NULL,
  `TRADE_AGENT` varchar(30) DEFAULT NULL,
  `GMT_MODIFIED` datetime DEFAULT NULL,
  `FP_TIME_OUT_RULE` varchar(1000) DEFAULT NULL,
  `FOREX_RATE_ID` bigint(11) DEFAULT NULL,
  `FOREX_CURRENCY` varchar(8) DEFAULT NULL,
  `FOREX_TOTAL_FEE` bigint(15) DEFAULT NULL,
  `GOODS_INNER_SELL_RATE` int(3) DEFAULT NULL,
  `PREPARE_COUPON_FEE` int(15) DEFAULT NULL,
  `PREPARE_COUPON_LIST` varchar(64) DEFAULT NULL,
  `FOREX_CERT_NO` varchar(128) DEFAULT NULL,
  `GMT_CREATE` datetime DEFAULT NULL,
  `AUTH_DEBIT_TYPE` varchar(1) DEFAULT NULL,
  `SUPPORT_CREDIT_CARD` varchar(256) DEFAULT NULL,
  `SPECIFIED_PAY_CHANNEL` varchar(128) DEFAULT NULL,
  `CHARGE_CHANNEL_INFO` varchar(256) DEFAULT NULL,
  `PAY_CHANNELS` varchar(900) DEFAULT NULL,
   unique KEY (`TRADE_NO`)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `trade_fund_bill` (
  `BILL_NO` varchar(64) NOT NULL,
  `TRADE_NO` varchar(64) NOT NULL,
  `PAY_ACCOUNT_NO` varchar(20) NOT NULL,
  `RECEIVE_ACCOUNT_NO` varchar(20) NOT NULL,
  `CURRENCY` varchar(3) NOT NULL DEFAULT '156',
  `AMOUNT` bigint(15) NOT NULL,
  `TYPE` char(2) NOT NULL,
  `STATUS` char(1) NOT NULL,
  `CHANNEL` char(2) NOT NULL,
  `GMT_PAY` datetime DEFAULT '2000-00-00 00:00:00',
  `IW_TRANS_LOG_ID` bigint(12) DEFAULT NULL,
  `GMT_TRADE_CREATE` datetime NOT NULL,
  `GMT_CREATE` datetime NOT NULL,
  `GMT_MODIFIED` datetime NOT NULL,
  `MEMO` varchar(1000) DEFAULT NULL,
  `BANK_PAY_ONLINE_ID` bigint(11) DEFAULT NULL,
  `BANK_AMOUNT` bigint(15) DEFAULT NULL,
  `BANK_TYPE` varchar(8) DEFAULT NULL,
  `FREEZE_TYPE` varchar(5) DEFAULT NULL,
  `PARTNER_ID` varchar(30) DEFAULT NULL,
  `OUT_ORDER_NO` varchar(64) DEFAULT NULL,
  `DEPOSIT_ID` varchar(16) DEFAULT NULL,
  `BILL_EXT` varchar(200) DEFAULT NULL,
  `id` bigint(11) NOT NULL AUTO_INCREMENT,
  PRIMARY KEY (`id`),
  key byd_trade_fund_bill_bno_ind  (bill_no),
  key byd_trade_fund_bill_pf_ind   (pay_account_no, gmt_trade_create),
  key byd_trade_fund_bill_rf_ind   (receive_account_no, gmt_trade_create),
  key trade_fund_bill_ptg_ind      (partner_id, gmt_trade_create)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `trade_goods_info` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT,
  `TRADE_NO` varchar(64) NOT NULL,
  `GOODS_ID` varchar(32) NOT NULL DEFAULT ' ',
  `GOODS_TITLE` varchar(256) DEFAULT NULL,
  `GOODS_BID` bigint(18) NOT NULL DEFAULT '0',
  `GOODS_QUANTITY` bigint(11) NOT NULL DEFAULT '1',
  `DETAIL_URL` varchar(400) DEFAULT NULL,
  `GOODS_MEMO` varchar(1000) DEFAULT NULL,
  `REFER_URL` varchar(400) DEFAULT NULL,
  `PARTNER` varchar(100) DEFAULT NULL,
  `GMT_CREATE` datetime NOT NULL,
  `OTHER_FEE` bigint(15) DEFAULT NULL,
  `CATEGORY` varchar(24) DEFAULT NULL,
  `PROMOTION_DESC` varchar(256) DEFAULT NULL,
  `PIC_URL` varchar(400) DEFAULT NULL,
  PRIMARY KEY (`id`),
  key trade_goods_info_tno (trade_no)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `trade_logistics` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT,
  `TRADE_NO` varchar(64) NOT NULL,
  `LOGISTICS_ID` varchar(20) DEFAULT NULL,
  `TRANSPORT_TYPE` int(2) DEFAULT NULL,
  `TRANSPORT_FEE` bigint(15) NOT NULL,
  `OTHER_FEE` bigint(15) NOT NULL,
  `ADDITIONAL_SERVICE` varchar(300) DEFAULT NULL,
  `LOGISTICS_STATUS` varchar(100) DEFAULT NULL,
  `SEND_FULLNAME` varchar(256) DEFAULT NULL,
  `SEND_PHONE` varchar(30) DEFAULT NULL,
  `SEND_MOBILE_PHONE` varchar(30) DEFAULT NULL,
  `SEND_ADDRESS` varchar(256) DEFAULT NULL,
  `SEND_POST` varchar(20) DEFAULT NULL,
  `RECEIVE_FULLNAME` varchar(256) DEFAULT NULL,
  `RECEIVE_PHONE` varchar(30) DEFAULT NULL,
  `RECEIVE_MOBILE_PHONE` varchar(30) DEFAULT NULL,
  `RECEIVE_ADDRESS` varchar(256) DEFAULT NULL,
  `RECEIVE_POST` varchar(20) DEFAULT NULL,
  `LOGISTICS_MEMO` varchar(600) DEFAULT NULL,
  `TRADE_OR_REFUND_FLAG` int(2) NOT NULL,
  `GMT_CREATE` datetime DEFAULT NULL,
  `GMT_APPLY` datetime DEFAULT NULL,
  `GMT_FETCH` datetime DEFAULT NULL,
  `GMT_TRANSPORT` datetime DEFAULT NULL,
  `GMT_SIGN` datetime DEFAULT NULL,
  `GMT_MODIFIED` datetime DEFAULT NULL,
  `LOGISTICS_NAME` varchar(128) DEFAULT NULL,
  `INVOICE_NO` varchar(64) DEFAULT NULL,
  `SIGN_VOUCHER_NO` varchar(64) DEFAULT NULL,
  `SEND_IM_TYPE` varchar(10) DEFAULT NULL,
  `SEND_IM_ID` varchar(100) DEFAULT NULL,
  `RECEIVE_IM_TYPE` varchar(10) DEFAULT NULL,
  `RECEIVE_IM_ID` varchar(100) DEFAULT NULL,
  `TRANSPORT_PAYMENT` int(1) DEFAULT NULL,
  `TRANSPORT_MODIFIABLE` int(1) DEFAULT NULL,
  `SEND_ADDRESS_CODE` varchar(8) DEFAULT NULL,
  `RECEIVE_ADDRESS_CODE` varchar(8) DEFAULT NULL,
  `TRANSPORT_TYPE_SEND` int(2) DEFAULT NULL,
  PRIMARY KEY (`id`),
   key trade_logistics_tt_u (trade_no, trade_or_refund_flag)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `trade_process` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT primary key,
  `TRADE_NO` varchar(64) NOT NULL DEFAULT ' ',
  `OUT_TRADE_NO` varchar(64) DEFAULT NULL,
  `RECEIVE_FEE` bigint(15) NOT NULL DEFAULT '0',
  `SEND_FEE` bigint(15) NOT NULL DEFAULT '0',
  `ADJUST_FEE` bigint(15) NOT NULL DEFAULT '0',
  `MEMO` varchar(600) DEFAULT NULL,
  `OPERATE_TIME` datetime DEFAULT NULL,
  `OPERATE_NAME` varchar(256) DEFAULT NULL,
  `FLAG_REFUND` varchar(4) NOT NULL DEFAULT '0',
  `FLAG_DEFERPAY` varchar(4) NOT NULL DEFAULT '0',
  `FLAG_PAYMENT` varchar(8) NOT NULL DEFAULT '0',
  `FLAG_TRADE_LOCKED` varchar(4) NOT NULL DEFAULT '0',
  `FLAG_SELLER_MDF` varchar(4) NOT NULL DEFAULT '0',
  `GMT_CREATE` datetime NOT NULL DEFAULT '2000-00-00 00:00:00',
  `GMT_MODIFIED` datetime DEFAULT NULL,
  `GMT_RECEIVE_PAY` datetime DEFAULT NULL,
  `GMT_SEND_PAY` datetime DEFAULT NULL,
  `GMT_SEND_GOODS` datetime DEFAULT NULL,
  `TIME_OUT_RULE` varchar(1000) DEFAULT NULL,
  `GMT_OUT_TRADE_NO_CREATED` datetime DEFAULT NULL,
  `COUPON_CLEAR_FEE` bigint(15) DEFAULT NULL,
  `COUPON_ADJUST_FEE` bigint(15) DEFAULT NULL,
  `REFUND_FEE` bigint(15) DEFAULT NULL,
  `GMT_REFUND_PAY` datetime DEFAULT NULL,
  `GMT_FINISH` datetime DEFAULT NULL,
  `ACCOUNT_TX_ID` varchar(40) DEFAULT NULL,
  `ADJUST_TIMEOUT` bigint(30) DEFAULT NULL,
  `PRE_AUTH_FEE` bigint(15) DEFAULT NULL,
  `GMT_PRE_AUTH` datetime DEFAULT NULL,
  `ACTUAL_ADJUST_FEE` bigint(15) DEFAULT NULL,
  `EXPIRE_TIME` datetime DEFAULT NULL,
  unique KEY (`TRADE_NO`),
  key ind_byd_trd_process_grpay (gmt_receive_pay, receive_fee)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `trade_outtrade` (
  `PARTNER_ID` varchar(200) NOT NULL,
  `OUT_TRADE_NO` varchar(64) NOT NULL,
  `TRADE_FROM` varchar(100) NOT NULL,
  `TRADE_NO` varchar(64) NOT NULL,
  `GMT_CREATE` datetime NOT NULL,
  `id` bigint(11) NOT NULL AUTO_INCREMENT,
  PRIMARY KEY (`id`),
  key trade_outtrade_otn (out_trade_no, partner_id, trade_from)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `trade_note` (
  `id` bigint(11) NOT NULL AUTO_INCREMENT,
  `TRADE_NO` varchar(64) NOT NULL,
  `TRADE_STATUS` varchar(64) DEFAULT ' ',
  `BUYER_EMAIL` varchar(100) NOT NULL DEFAULT ' ',
  `SELLER_EMAIL` varchar(100) NOT NULL DEFAULT ' ',
  `OPERATOR_EMAIL` varchar(100) NOT NULL DEFAULT ' ',
  `OPERATOR_IP` varchar(15) DEFAULT NULL,
  `GMT_CREATE` datetime DEFAULT '2000-01-01 00:00:00',
  `OPERATOR_MEMO` varchar(256) DEFAULT NULL,
  `NOTE_TYPE` varchar(100) DEFAULT NULL,
  `REMOTE_MAC` varchar(64) DEFAULT NULL,
  PRIMARY KEY (`id`),
  unique key uk_trade_note_tn(trade_no, id, gmt_create)
)ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
CREATE TABLE `sequence` (
  `name` varchar(32) NOT NULL DEFAULT '' COMMENT '',
  `value` bigint(20) DEFAULT NULL COMMENT '',
  PRIMARY KEY (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=gbk]])

   con:query([[
INSERT INTO sequence(name,value) VALUES('seq_trade_no',1)
   ]])
end

-- Generate a new trade ID (trade_no)
function new_trade_no()
   return string.format("2008%s%s%s%s%s%s%s%s%s%s%s%s001",
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000),
                        sysbench.rand.default(0, 5000))
end

-- Generate a random string of alphabetical characters with a given length (the
-- StrRand{N} field)
function StrRand(len)
   return sysbench.rand.string(string.rep("@", len))
end

-- Generate a fixed string with a given length (the String{N} field)
function String(len)
   return string.rep("X", len)
end

-- Generate a random string of numeric characters with a given length
function StrRandNum(len)
   return sysbench.rand.string(string.rep("#", len))
end

-- Generate a random number in a given range
function NumRand(a, b)
   return sysbench.rand.default(a,b)
end

-- Return the "NOW()" string to implement Date{'now()'}
function Date()
   return "NOW()"
end

-- Insert a new row into the `trade_base` table with a given trade_no ID
function insert_into_trade_base(trade_no, out_trade_no)
   con:query(string.format([[INSERT INTO trade_base VALUES
(NULL, '%s', '%s', '%s', '%s', '%s', '%s', %d, %d, '%s', '%s', '%s', '%s', %d,
%d, %d, '%s', '%s', '%s', %s, '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s',
'%s', %s, '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', %d, '%s')]],
               out_trade_no,        -- OUT_TRADE_NO#StrRand{19};
               trade_no,        -- TRADE_NO#Func{'gettradeno'};
               String(20),        -- SELLER_ACCOUNT#String{20};
               String(19),        -- SELLER_LOGIN_EMAIL#String{19};
               String(20),        -- BUYER_ACCOUNT#String{20};
               String(17),        -- BUYER_LOGIN_EMAIL#String{17};
               NumRand(1,9),        -- SELLER_TYPE#NumRand{1,9};
               NumRand(1,9),        -- BUYER_TYPE#???
               String(4),        -- TRADE_FROM#String{4};
               String(20),        -- TRADE_FROM#String{4};
               String(1),        -- OPERATOR_ROLE#String{1};
               String(15),        -- TRADE_STATUS#String{15};
               NumRand(100,3000),    -- TOTAL_FEE#NumRand{100,3000};
               NumRand(1,9),        -- SERVICE_FEE_RATIO#NumRand{1,9};
               NumRand(1,9),        -- SERVICE_FEE#NumRand{1,9};
               String(3),        -- CURRENCY#String{3};
               String(11),        -- SELLER_ACTION#String{11};
               String(16),        -- BUYER_ACTION#String{16};
               Date(),            -- GMT_CREATE#Date{'now()'};
               String(16),        -- SELLER_USER_ID#String{16};
               String(16),        -- BUYER_USER_ID#String{16};
               String(19),        -- ADDITIONAL_TRD_STATUS#String{19};
               String(1),        -- TRADE_TYPE#String{1};
               String(7),        -- SELLER_FULLNAME#String{7};
               String(4),        -- BUYER_FULLNAME#String{4};
               String(8),        -- SELLER_NICK#String{8};
               String(7),        -- BUYER_NICK#String{7};
               String(28),        -- GOODS_TITLE#String{28};
               Date(),            -- GMT_LAST_MODIFIED_DT#Date{'now()'};
               String(4),        -- STOP_TIMEOUT#???
               String(4),        -- GATHERING_TYPE#???
               String(1),        -- BUYER_MARKER#???
               String(1),        -- SELLER_MARKER#???
               String(14),        -- BUYER_MARKER_MEMO#String{14};
               String(14),        -- SELLER_MARKER_MEMO#String{14};
               String(10),        -- CHANNEL#String{10};
               String(10),        -- PRODUCT#String{10};
               String(19),        -- PAY_CHANNEL#String{19};
               NumRand(1,9),        -- OTHER_SERVICE_FEE#NumRand{1,9}
               String(64)        -- RELATION_PRO#???
   ))
end

-- Insert a new row into the `trade_note` table with a given trade_no ID
function insert_into_trade_note(trade_no)
   con:query(string.format([[INSERT INTO trade_note VALUES
(NULL, '%s', '%s', '%s', '%s', '%s', '%s', %s, '%s', '%s', '%s')]],
      trade_no,            -- TRADE_NO#Func{'gettradeno'};
      String(17),        -- TRADE_STATUS#String{17};
      String(16),        -- BUYER_EMAIL#String{16};
      String(18),        -- SELLER_EMAIL#String{18};
      String(18),        -- OPERATOR_EMAIL#String{18};
      String(12),        -- OPERATOR_IP#String{12};
      Date(),            -- GMT_CREATE#Date{'now()'};
      String(77),        -- OPERATOR_MEMO#String{77};
      String(20),        -- NOTE_TYPE#String{20};
      String(12)        -- REMOTE_MAC#String{12}
   ))
end

-- Insert a new row into the `timeout` table with a given trade_no ID
function insert_into_timeout(trade_no)
   con:query(string.format([[INSERT INTO timeout VALUES
(NULL, '%s', '%s', '%s', '%s', %s, '%s', '%s', '%s', '%s', %s, %d, %s)]],
      String(32),        -- JOB_ID#StrRand{32};
      trade_no,            -- TRADE_NO#Func{'gettradeno'};
      String(16),        -- USER_ID#String{16};
      String(1),        -- PAYTYPE#String{1};
      Date(),            -- END_TIME#Date{'now()'};
      String(18),        -- ACTION#String{18};
      String(28),        -- PARAMETER#String{28};
      String(1),        -- STATUS#String{1};
      String(1),        -- ACTION_TYPE#String{1};
      Date(),             -- GMT_CREATE#Date{'now()'};
      NumRand(1,9),        -- PRIOR_LEVEL#NumRand{1,9};
      Date()            -- GMT_MODIFIED#Date{'now()'}
   ))
end

-- Insert a new row into the `trade_logistics` table with a given trade_no ID
function insert_into_trade_logistics(trade_no)
   con:query(string.format([[INSERT INTO trade_logistics VALUES
(NULL, '%s', '%s', %d, %d, %d, '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s',
'%s', '%s', '%s', '%s', '%s', %d, %s, %s, %s, %s, %s, %s, '%s', '%s', '%s',
'%s', '%s', '%s', '%s', %d, %d, '%s', '%s', %d)]],
      trade_no,            -- TRADE_NO#Func{'gettradeno'};
      String(6),        -- LOGISTICS_ID#String{6};
      NumRand(1,9),        -- TRANSPORT_TYPE#NumRand{1,9};
      NumRand(1,99),        -- TRANSPORT_FEE#NumRand{1,99};
      NumRand(1,9),        -- OTHER_FEE#NumRand{1,9};
      String(300),        -- ADDITIONAL_SERVICE#???
      String(18),        -- LOGISTICS_STATUS#String{18};
      String(4),        -- SEND_FULLNAME#???
      String(13),        -- SEND_PHONE#???
      String(11),        -- SEND_MOBILE_PHONE#???
      String(16),        -- SEND_ADDRESS#???
      String(6),        -- SEND_POST#???
      String(4),        -- RECEIVE_FULLNAME#String{4};
      String(13),        -- RECEIVE_PHONE#String{13};
      String(11),        -- RECEIVE_MOBILE_PHONE#String{11};
      String(16),        -- RECEIVE_ADDRESS#String{16};
      String(6),        -- RECEIVE_POST#String{6};
      String(18),        -- LOGISTICS_MEMO#String{18};
      NumRand(1,9),        -- TRADE_OR_REFUND_FLAG#NumRand{1,9};
      Date(),            -- GMT_CREATE#Date{'now()'};
      Date(),            -- GMT_APPLY#???
      Date(),            -- GMT_FETCH#???
      Date(),            -- GMT_TRANSPORT#Date{'now()'};
      Date(),            -- GMT_SIGN#Date{'now()'};
      Date(),            -- GMT_MODIFIED#Date{'now()'};
      String(4),        -- LOGISTICS_NAME#String{4};
      String(9),        -- INVOICE_NO#String{9};
      String(16),        -- SIGN_VOUCHER_NO#String{16};
      String(10),        -- SEND_IM_TYPE#???
      String(10),        -- SEND_IM_ID#???
      String(10),        -- RECEIVE_IM_TYPE#???
      String(10),        -- RECEIVE_IM_ID#???
      NumRand(1,9),        -- TRANSPORT_PAYMENT#NumRand{1,9};
      NumRand(1,8),        -- TRANSPORT_MODIFIABLE#NumRand{1,8};
      String(5),        -- SEND_ADDRESS_CODE#???;
      String(5),        -- RECEIVE_ADDRESS_CODE#String{5};
      NumRand(1,9)        -- TRANSPORT_TYPE_SEND#NumRand{1,9}
   ))
end

-- Insert a new row into the `trade_process` table with a given trade_no ID
function insert_into_trade_process(trade_no)
   con:query(string.format([[INSERT INTO trade_process VALUES
(NULL, '%s', '%s', %d, %d, %d, '%s', %s, '%s', '%s', '%s', '%s', '%s', '%s', %s,
%s, %s, %s, %s, '%s', %s, %d, %d, %d, %s, %s, '%s', %d, %d, %s, %d, %s)]],
      trade_no,            -- TRADE_NO#Func{'gettradeno'};
      String(19),        -- OUT_TRADE_NO#String{19};
      NumRand(100, 3000),    -- RECEIVE_FEE#NumRand{100,3000};
      NumRand(100, 10000),    -- SEND_FEE#NumRand{100,10000};
      NumRand(1, 99),        -- ADJUST_FEE#NumRand{1,99};
      String(94),        -- MEMO#String{94};
      Date(),            -- OPERATE_TIME#Date{'now()'};
      String(4),        -- OPERATE_NAME#???
      String(2),        -- FLAG_REFUND#String{2};
      String(2),        -- FLAG_DEFERPAY#String{2};
      String(2),        -- FLAG_PAYMENT#String{2};
      String(2),        -- FLAG_TRADE_LOCKED#String{2};
      String(2),        -- FLAG_SELLER_MDF#String{2};
      Date(),            -- GMT_CREATE#Date{'now()'};
      Date(),            -- GMT_MODIFIED#Date{'now()'};
      Date(),            -- GMT_RECEIVE_PAY#Date{'now()'};
      Date(),            -- GMT_SEND_PAY#Date{'now()'};
      Date(),            -- GMT_SEND_GOODS#Date{'now()'};
      String(27),        -- TIME_OUT_RULE#String{27};
      Date(),            -- GMT_OUT_TRADE_NO_CREATED#???
      NumRand(1,9),        -- COUPON_CLEAR_FEE#NumRand{1,9};
      NumRand(1,9),        -- COUPON_ADJUST_FEE#NumRand{1,9};
      NumRand(1,9),        -- REFUND_FEE#NumRand{1,9};
      Date(),            -- GMT_REFUND_PAY#Date{'now()'};
      Date(),            -- GMT_FINISH#Date{'now()'};
      String(40),        -- ACCOUNT_TX_ID#???
      NumRand(1,10),        -- ADJUST_TIMEOUT#NumRand{1,10};
      NumRand(1,9),        -- PRE_AUTH_FEE#NumRand{1,9};
      Date(),            -- GMT_PRE_AUTH#Date{'now()'};
      NumRand(1,9),        -- ACTUAL_ADJUST_FEE#NumRand{1,9};
      Date()            -- EXPIRE_TIME#Date{'now()'}
   ))
end

-- Insert a new row into the `trade_good_info` table with a given trade_no ID
function insert_into_trade_goods_info(trade_no)
   con:query(string.format([[INSERT INTO trade_goods_info VALUES
(NULL, '%s', '%s', '%s', %d, %d, '%s', '%s', '%s', '%s', %s, %d, '%s', '%s',
'%s')]],
      trade_no,            -- TRADE_NO#Func{'gettradeno'};
      String(9),        -- GOODS_ID#String{9};
      String(28),        -- GOODS_TITLE#String{28};
      NumRand(3,3000),        -- GOODS_BID#NumRand{3,3000};
      NumRand(1,9),        -- GOODS_QUANTITY#NumRand{1,9};
      String(27),        -- DETAIL_URL#String{72};
      String(40),        -- GOODS_MEMO#String{40};
      String(10),        -- REFER_URL#String{10};
      String(20),        -- PARTNER#???
      Date(),            -- GMT_CREATE#Date{'now()'};
      NumRand(1,9),        -- OTHER_FEE#NumRand{1,9};
      NumRand(1,7),        -- CATEGORY#NumRand{7};
      String(7),        -- PROMOTION_DESC#String{7};
      String(92)        -- PIC_URL#String{92}
   ))
end

-- Insert a new row into the `trade_outtrade` table with given trade_no
-- and out_trade_no IDs
function insert_into_trade_outtrade(trade_no, out_trade_no)
   con:query(string.format([[INSERT INTO trade_outtrade VALUES
('%s', '%s', '%s', '%s', %s, NULL)]],
      String(18),        -- PARTNER_ID#String{18};
      out_trade_no,        -- OUT_TRADE_NO#Func{'getradeno'};
      String(4),        -- TRADE_FROM#String{4};
      trade_no,            -- TRADE_NO#Func{'gettradeno'};
      Date()            -- GMT_CREATE#Date{'now()'}
      ))
end

-- Insert a new row into the `exterface_invoke` table
function insert_into_exterface_invoke()
   con:query(string.format([[INSERT INTO exterface_invoke VALUES
(NULL, '%s', '%s', '%s', '%s', '%s', '%s', '%s', %s, %s, '%s', %s, %s)]],
      String(20),        -- PARTNER_ID#String{20};
      StrRand(16),        -- TARGET#StrRand{16};
      String(14),        -- EXTERFACE#String{14};
      String(42),        -- NOTIFY_URL#String{42};
      String(78),        -- RETURN_URL#String{78};
      String(3),        -- SIGN_TYPE#String{3};
      String(3),        -- CHARSET#String{3};
      Date(),            -- GMT_INVOKE#Date{'now()'};
      Date(),            -- GMT_FINISH#Date{'now()'};
      String(3),        -- TARGET_TYPE#String{3};
      Date(),            -- GMT_NOTIFY#Date{'now()'};
      Date()            -- GMT_RETURN#???
   ))
end

-- Insert a new row into the `fund_flow_info` table with a given order_no ID
function insert_into_fund_flow_info(order_no)
   con:query(string.format([[INSERT INTO fund_flow_info VALUES
(NULL, '%s', '%s', '%s', '%s', %d, '%s', '%s', %s, %s)]],
      order_no,            -- ORDER_NO#Func{'gettradeno'};
      String(20),        -- TRANS_ACCOUNT_OUT#String{20};
      String(20),        -- TRANS_ACCOUNT_IN#String{20};
      String(20),        -- ACC_TYPE#String{20};
      NumRand(78,99),        -- FUND_CLEAR_FEE#NumRand{78,99};
      String(7),        -- PAY_TOOL_TYPE#String{7};
      String(13),        -- PAY_TOOL_CONTEXT#String{13};
      Date(),            -- GMT_CREATE#Date{'now()'};
      Date()            -- GMT_MODIFIED#Date{'now()'};
   ))
end

-- Insert a new row into the `trade_fund_bill` table with a given trade_no ID
function insert_into_trade_fund_bill(trade_no)
   con:query(string.format([[INSERT INTO trade_fund_bill VALUES
('%s', '%s', '%s', '%s', '%s', %u, '%s', '%s', '%s', %s, %d, %s, %s, %s, '%s',
%d, %d, '%s', '%s', '%s', '%s', '%s', '%s', NULL)]],
      String(26),        -- BILL_NO#String{26};
      trade_no,            -- TRADE_NO#Func{'gettradeno'};
      String(20),        -- PAY_ACCOUNT_NO#String{20};
      String(19),        -- RECEIVE_ACCOUNT_NO#String{19};
      String(3),        -- CURRENCY#String{3};
      NumRand(1000,9999),    -- AMOUNT#NumRand{1000,9999};
      String(2),        -- TYPE#String{2};
      String(1),        -- STATUS#String{1};
      String(2),        -- CHANNEL#String{2};
      Date(),            -- GMT_PAY#Date{'now()'};
      NumRand(10000,999999),    -- IW_TRANS_LOG_ID#NumRand{10000,999999};
      Date(),            -- GMT_TRADE_CREATE#Date{'now()'};
      Date(),            -- GMT_CREATE#Date{'now()'};
      Date(),            -- GMT_MODIFIED#Date{'now()'};
      String(41),        -- MEMO#String{41};
      NumRand(453,99999),    -- BANK_PAY_ONLINE_ID#NumRand{453,99999};
      NumRand(20,99),        -- BANK_AMOUNT#NumRand{20,99};
      String(4),        -- BANK_TYPE#String{4};
      String(5),        -- FREEZE_TYPE#String{5};
      String(19),        -- PARTNER_ID#String{19};
      String(19),        -- OUT_ORDER_NO#String{19};
      String(16),        -- DEPOSIT_ID#String{16};
      String(78)        -- BILL_EXT#String{78};
   ))
end

-- Insert a new row into the `trade_ext` table with a given trade_no ID
function insert_into_trade_ext(trade_no)
   con:query(string.format([[INSERT INTO trade_ext VALUES
(NULL, '%s', '%s', '%s', %s, '%s', %d, '%s', %d, %d, %d, '%s', '%s', %s, '%s',
'%s', '%s', '%s', '%s')]],
      trade_no,         -- TRADE_NO#Func{'gettradeno'};
      String(58),        -- EXT_INFO#String{58};
      String(6),        -- TRADE_AGENT#String{6};
      Date(),             -- GMT_MODIFIED#Date{'now()'};
      String(27),        -- FP_TIME_OUT_RULE#???
      NumRand(1,9),        -- FOREX_RATE_ID#NumRand{1,9};
      String(3),        -- FOREX_CURRENCY#String{3};
      NumRand(1,9),        -- FOREX_TOTAL_FEE#NumRand{1,9};
      NumRand(1,9),        -- GOODS_INNER_SELL_RATE#NumRand{1,9};
      NumRand(1,9),        -- PREPARE_COUPON_FEE#NumRand{1,9};
      String(64),        -- PREPARE_COUPON_LIST#???
      String(38),        -- FOREX_CERT_NO#String{38};
      Date(),            -- GMT_CREATE#Date{'now()'};
      String(1),        -- AUTH_DEBIT_TYPE#String{1};
      String(2),        -- SUPPORT_CREDIT_CARD#String{2};
      String(9),        -- SPECIFIED_PAY_CHANNEL#String{9};
      String(16),        -- CHARGE_CHANNEL_INFO#String{16};
      String(19)        -- PAY_CHANNELS#???
   ))
end

-- Insert a new row into the `notify` table
function insert_into_notify()
   con:query(string.format([[INSERT INTO notify VALUES
(NULL, '%s', '%s', %d, '%s', %s, %s, '%s', '%s', '%s', '%s', '%s', %s)]],
      String(23),        -- PLUGIN#String{23};
      String(256),        -- PARAMETER#String{256};
      NumRand(1,9),        -- GROUP_NO#NumRand{1,9};
      String(1),        -- STATUS#String{1};
      Date(),            -- GMT_CREATE#Date{'now()'};
      Date(),            -- GMT_MODIFY#Date{'now()'};
      String(16),        -- IW_USER_ID#String{16};
      String(27),        -- CHANNEL_ENDPOINT#String{27};
      String(16),        -- BUSINESS_ORDER#String{16};
      String(33),        -- BUSINESS_TYPE#String{33};
      String(7),        -- PLUGIN_HOST#String{7};
      Date()            -- GMT_LOCAL_CREATE#Date{'now()'};
   ))
end

function sysbench.hooks.before_restart_event(err)
  con:query("ROLLBACK")
end

function sysbench.hooks.report_intermediate(stat)
-- --   print("my stat: ", val)
   if  sysbench.opt.report_csv == "yes" then
       sysbench.report_csv(stat)
   else
       sysbench.report_default(stat)
   end
end