#!/usr/bin/env sysbench

-- This program is free software; you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation; either version 2 of the License, or
-- (at your option) any later version.

-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

-- ----------------------------------------------------------------------
-- TPCC-like workload
-- ----------------------------------------------------------------------

require("tpcc_common")
require("tpcc_run_grv")
require("tpcc_check")

function thread_init(thread_id)

    drv = sysbench.sql.driver()
    con = drv:connect()
    replica_con = {}
    
    con:query("set session tx_isolation = 'repeatable-read'") 

    if drv:name() == "mysql" then 
      con:query("SET autocommit=0")
    end

    if sysbench.opt.GRV_scope == 'cluster' then
      rs = con:query("select slave_host, slave_port from information_schema.INNODB_LOG_READER where role = 'replica' and status = 'active'")

      replica_count = rs.nrows
      for i = 1, replica_count do
        row = rs:fetch_row()
        local slave_host = tostring(row[1])
        local slave_port = tonumber(row[2])
        replica_con[i] = ffi.C.db_connection_create2(drv, slave_port)

        if replica_con[i] == nil then
          print("Connection to replica failed")
          os.exit()
        end

        replica_con[i]:query("set session tx_isolation = 'read-committed'") 

        if drv:name() == "mysql" then 
          replica_con[i]:query("SET autocommit=0")
        end
    
      end
    else
      replica_count = 100
      for i = 1, replica_count do
        replica_con[i] = drv:connect()

        if replica_con[i] == nil then
          print("Connection to local failed")
          os.exit()
        end

        replica_con[i]:query("set session tx_isolation = 'read-committed'") 

        if drv:name() == "mysql" then 
          replica_con[i]:query("SET autocommit=0")
        end
    
      end
    end

    timed_event_init()
end

function event()
  local i
  local port = con:query_row("select @@port")
  --print(string.format("port: %s\n", port))
  for i = 1, replica_count do
    port = replica_con[i]:query_row("select @@port")
    --print(string.format("port: %s\n", port))
  end

-- print( NURand (1023,1,3000))
  if sysbench.opt.read_only==1 then
    local max_trx =  2
    local trx_type = sysbench.rand.uniform(1,max_trx)
    if trx_type <= 1 then
      trx="orderstatus"
    elseif trx_type <= 2 then
      trx="stocklevel"
    end
    _G[trx]()
  else
    local max_trx =  sysbench.opt.enable_purge == "yes" and 24 or 23
    local trx_type = sysbench.rand.uniform(1,max_trx)
    if trx_type <= 10 then
      trx="new_order"
    elseif trx_type <= 20 then
      trx="payment"
    elseif trx_type <= 21 then
      trx="orderstatus"
    elseif trx_type <= 22 then
      trx="delivery"
    elseif trx_type <= 23 then
      trx="stocklevel"
    elseif trx_type <= 24 then
      trx="purge"
    end
    _G[trx]()
  end
  throttle_event() --Calculate the time delay to achieve target TPS
end

function sysbench.hooks.before_restart_event(err)
  local i
  con:query("ROLLBACK")
  if errdesc.sql_errno == 2013 or -- CR_SERVER_LOST
     errdesc.sql_errno == 2055 or -- CR_SERVER_LOST_EXTENDED
     errdesc.sql_errno == 2006 or -- CR_SERVER_GONE_ERROR
     errdesc.sql_errno == 2011    -- CR_TCP_CONNECTION
  then
    for i = 1, replica_count do
      local rc = ffi.C.db_connection_reconnect(replica_con[i])
      if rc ~= 0 then
        print("Reconnect to replica failed, removing it from list")
        table.remove(replica_con, i)
        return
      end

      replica_con[i]:query("set session tx_isolation = 'read-committed'") 

      if drv:name() == "mysql" then 
        replica_con[i]:query("SET autocommit=0")
      end
      print(replica_con[i]:query_row("select @@port") )

    end
  end
end

function sysbench.hooks.report_intermediate(stat)
-- --   print("my stat: ", val)
   if  sysbench.opt.report_csv == "yes" then
   	sysbench.report_csv(stat)
   else
   	sysbench.report_default(stat)
   end
end


-- vim:ts=4 ss=4 sw=4 expandtab
